import QtQuick 2.4
import Components 1.0
import WorkoutRunData 1.0
import WorkoutManager 1.0
import "string_format.js" as StrFormat

Item {
    id: root

    property int tableWidth: 1000 * main.hScaleFactor
    property int tableHMargin: 40 * main.hScaleFactor
    property int numColumns: 5 // dataTable.models.length
    property int columnWidth: (tableWidth)/numColumns
    property alias selectable: dataTable.selectable
    property alias dataDisplayModel: dataTable.dataDisplayModel
    property alias selectedCount: dataTable.selectedCount

    // build data set for chart
    Component.onCompleted: {
        reset();
        updateDataValues();
    }

    function convertUnits(speedMiles) {
        return workoutScreen.unitState === "METRIC" ? Globals.mileToKm(speedMiles) : speedMiles;
    }

    function convertMeters(feet) {
        return workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(feet) : feet;
    }

    function convertPaceTime(pace) {
        return workoutScreen.unitState === "METRIC" ? pace/1.61 : pace;
    }

    function updateDataValues() {
        distanceColumn.setProperty(0, "value", Math.floor(aWorkoutRunData.totalFloorsClimbed).toFixed(0));
        distanceColumn.setProperty(1, "value", Math.floor(aWorkoutRunData.totalStepsClimbed).toFixed(0));
        distanceColumn.setProperty(2, "value", convertUnits(aWorkoutRunData.totalDistance).toFixed(2));
        distanceColumn.setProperty(3, "value", convertUnits(aWorkoutRunData.distanceRemaining).toFixed(2));
        distanceColumn.setProperty(4, "value", convertMeters(aWorkoutRunData.totalFeetClimbed).toFixed(0));
        distanceColumn.setProperty(5, "value", aWorkoutRunData.avgIncline.toFixed(2));

        timeColumn.setProperty(0, "value", StrFormat.formatTimeStringMinAndSec(aWorkoutRunData.totalTime));
        timeColumn.setProperty(1, "value", workoutProgram.isMonumentWorkout && aWorkoutRunData.mode === WorkoutRunData.WORKOUT_MODE
                               ? workoutManager.treadIncline <= 0 ? "--:--" : StrFormat.formatTimeStringMinAndSec(aWorkoutRunData.climbTimeRemaining)
                               : StrFormat.formatTimeStringMinAndSec(aWorkoutRunData.timeRemaining))
        timeColumn.setProperty(2, "value", main.isTreadmill || main.isStepmill
                               ? StrFormat.formatTimeStringMinAndSec(convertPaceTime(aWorkoutRunData.treadPace))
                               : aWorkoutRunData.rpm.toFixed(0));
        timeColumn.setProperty(3, "value", main.isTreadmill || main.isStepmill
                               ? convertUnits(aWorkoutRunData.totalAverageSpeed).toFixed(1)
                               : aWorkoutRunData.totalAveragePace.toFixed(0)); // Avg RPM
        timeColumn.setProperty(4, "value", main.isTreadmill || main.isStepmill
                               ? convertUnits(aWorkoutRunData.totalMaximumSpeed).toFixed(1)
                               : aWorkoutRunData.totalMaximumPace.toFixed(0)); // max RPM

        caloriesColumn.setProperty(0, "value", aWorkoutRunData.totalCalories.toFixed(0));
        caloriesColumn.setProperty(1, "value", aWorkoutRunData.calsPerHour.toFixed(0));

        metsColumn.setProperty(0, "value", aWorkoutRunData.mets.toFixed(0));
        metsColumn.setProperty(1, "value", aWorkoutRunData.watts.toFixed(0));

        heartRateColumn.setProperty(0, "value", heartRateManager.heartRate.toFixed(0));
        heartRateColumn.setProperty(1, "value", aWorkoutRunData.totalMaximumHeartRate > 0
                                    ? aWorkoutRunData.totalMaximumHeartRate.toString()
                                    : "--");
        heartRateColumn.setProperty(2, "value", aWorkoutRunData.totalAverageHeartRate > 0
                                    ? aWorkoutRunData.totalAverageHeartRate.toString()
                                    : "--");
        heartRateColumn.setProperty(3, "value", workoutProgram.displayDataVars.targetHRAvailable
                                    ? aWorkoutRunData.targetHeartRate.toString()
                                    : "--");

        dataTable.updateDataDisplay();
    }

    function reset() {

        distanceColumn.setProperty(0, "isDisplayed", main.isStepmill);
        distanceColumn.setProperty(1, "isDisplayed", main.isStepmill);
        distanceColumn.setProperty(2, "isDisplayed", workoutProgram.displayDataVars.distanceAvailable);
        distanceColumn.setProperty(3, "isDisplayed", workoutProgram.displayDataVars.distanceRemainingAvailable);
        distanceColumn.setProperty(4, "isDisplayed", workoutProgram.displayDataVars.feetClimbedAvailable);
        distanceColumn.setProperty(5, "isDisplayed", main.isTreadmill);

        timeColumn.setProperty(0, "isDisplayed", workoutProgram.displayDataVars.timeAvailable);
        timeColumn.setProperty(1, "isDisplayed", workoutProgram.displayDataVars.timeRemainingAvailable);
        timeColumn.setProperty(2, "isDisplayed", workoutManager.workoutProgram.productType === WorkoutProgram.TREADMILL && workoutProgram.displayDataVars.paceAvailable);
        timeColumn.setProperty(3, "isDisplayed", workoutProgram.displayDataVars.paceAvailable); // AVG SPEED
        timeColumn.setProperty(4, "isDisplayed", false); // MAX SPEED

        caloriesColumn.setProperty(0, "isDisplayed", workoutProgram.displayDataVars.caloriesAvailable);
        caloriesColumn.setProperty(1, "isDisplayed", workoutProgram.displayDataVars.calsPerHourAvailable);

        metsColumn.setProperty(0, "isDisplayed", workoutProgram.displayDataVars.metsAvailable);
        metsColumn.setProperty(1, "isDisplayed", workoutProgram.displayDataVars.wattsAvailable);

        heartRateColumn.setProperty(0, "isDisplayed", workoutProgram.displayDataVars.heartRateAvailable);
        heartRateColumn.setProperty(1, "isDisplayed", workoutProgram.displayDataVars.heartRateAvailable); // MAX BPM
        heartRateColumn.setProperty(2, "isDisplayed", workoutProgram.displayDataVars.heartRateAvailable); // AVG BPM
        heartRateColumn.setProperty(3, "isDisplayed", workoutProgram.isHRCProgram);

        if(!selectable){
            distanceColumn.setProperty(0, "isDisplayed", main.isStepmill);
            distanceColumn.setProperty(1, "isDisplayed", main.isStepmill);
            distanceColumn.setProperty(2, "isDisplayed", workoutProgram.displayDataVars.distanceAvailable);
            distanceColumn.setProperty(3, "isDisplayed", false);
            distanceColumn.setProperty(4, "isDisplayed", workoutProgram.displayDataVars.feetClimbedAvailable);
            distanceColumn.setProperty(5, "isDisplayed", main.isTreadmill);
            timeColumn.setProperty(1, "isDisplayed", false);
            heartRateColumn.setProperty(0, "isDisplayed", false);
            heartRateColumn.setProperty(3, "isDisplayed", workoutProgram.displayDataVars.targetHRAvailable);

            return;
        }

        dataTable.reset();
    }

    ListModel {
        id: distanceColumn
        property string heading: qsTr("DISTANCE") + application.emptyString

        ListElement{ label: qsTr("FLOORS CLIMBED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("STEPS CLIMBED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("DISTANCE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("DISTANCE REMAINING"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("DISTANCE CLIMBED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("AVG INCLINE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
    }

    ListModel {
        id: timeColumn
        property string heading: qsTr("TIME") + application.emptyString

        ListElement{ label: qsTr("TIME ELAPSED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("TIME REMAINING"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("PACE"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("AVG SPEED"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("MAX SPEED"); isSelected: false; isDisplayed: true; value: "0"}
    }

    ListModel {
        id: caloriesColumn
        property string heading: qsTr("CALORIES") + application.emptyString

        ListElement{ label: qsTr("CALORIES"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("CALORIES/HR"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
    }

    ListModel {
        id: metsColumn
        property string heading: qsTr("METS") + application.emptyString

        ListElement{ label: qsTr("METS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("WATTS"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
    }

    ListModel {
        id: heartRateColumn
        property string heading: qsTr("HEART RATE") + application.emptyString

        ListElement{ label: qsTr("BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: true}
        ListElement{ label: qsTr("MAX BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("AVG BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
        ListElement{ label: qsTr("TARGET BPM"); isSelected: false; isDisplayed: true; value: "0"; isHR: false}
    }

    Connections {
        target: workoutManager
        onWorkoutDataChanged: {
            updateDataValues();
        }
    }

    Rectangle {
        id: tableWrap

        width: root.tableWidth + 2*root.tableHMargin
        height: root.height
        anchors.top: parent.top
        anchors.left: parent.left
        color: root.selectable ? main.colorToolbarBackground : "transparent"
        border.color: root.selectable ? "transparent" : main.colorUIBorder
        border.width: main.widthBorder

        SelectableDataTable {
            id: dataTable
            sourceFile: "DataSummaryTable"
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.leftMargin: root.tableHMargin
            width: root.tableWidth
            height: tableWrap.height
            models: [ distanceColumn, timeColumn, caloriesColumn, metsColumn, heartRateColumn ]
        }
    }
}
