import QtQuick 2.4
import Process 0.1
import "."
import "../Television"
import ".."
import "../Keypad"
import FileIO 1.0
import VLCVideo 1.0
import Components 1.0

Item {
    id: root
    property bool isActive: main.state === "DiagnosticState" && diagnosticsMain.currentSelectedIndex >= 0

    anchors.fill: parent

    Column {
        id: controlList
        spacing: 15 * main.vScaleFactor
        anchors.top: root.top
        anchors.topMargin: 20 * main.vScaleFactor
        anchors.left: root.left
        anchors.leftMargin: 20 * main.hScaleFactor
        width: root.width * 0.6
        height: root.height * 0.9

        Checkbox {
            id: cabEnabledToggle
            label: qsTr("C.A.B. Enabled?") + application.emptyString
            width: parent.width
            onPressed: {
                options.cabEnabled = value ? true : false
                if(value){
                    startTv()
                } else {
                    stopTv()
                }
            }
            Component.onCompleted: value = options.cabEnabled
        }
    }

    Rectangle {
        id: rightControlArea

        radius: 5 * main.scaleFactor
        width: root.width * 0.30
        height: previewTv.height + 230 * main.vScaleFactor
        color: "darkgrey"
        anchors {
            top: controlList.top
            right: parent.right
            rightMargin: 30 * main.hScaleFactor
        }

        Column {
            spacing: 15 * main.vScaleFactor
            anchors.fill: parent
            anchors.topMargin: 25 * main.vScaleFactor

            VLCSurface{
                id: previewTv
                player: tv.player
                aspect: tv.aspect
                playing: 1
                width: parent.width * 0.8
                height: width/tv.aspect
                anchors.horizontalCenter: parent.horizontalCenter
                onPlayingChanged: tv.on = playing
                z: 1000
            }

            StandardButton {
                id: channelUpButton

                width: 0.75 * parent.width
                height: 78 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter

                buttonText: qsTr("Channel Up") + application.emptyString
                onClicked: nucleus.cabChannelUp()
            }

            StandardButton {
                id: channelDownButton

                width: 0.75 * parent.width
                height: 78 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter

                buttonText: qsTr("Channel Down") + application.emptyString
                onClicked: nucleus.cabChannelDown()
            }
        }
    }

    Timer {
        id: tvRetryTimer
        interval: 1500
        running: true
        repeat: false
        onTriggered: {
            if (tv.on === false) {
                startTv()
            }
        }
    }

    function startTv() {
        /* Get the first item on the list */
        var item = tv.channelList.getChannelItem(0)

        /* Tune to preview item if valid */
        if(item !== null) {
            tv.tuneChannelID(item.id)
        }
    }

    function stopTv() {
        tv.stop()
    }

    Component.onDestruction: tv.on = false
}
