
import QtQuick 2.4
import Process 0.1
import "."
import "../Television"
import ".."
import "../Keypad"
import FileIO 1.0
import VLCVideo 1.0
import Components 1.0
import "../trueui"

Item {
    id: root
    property string tuner_id: tv.tunerType
    property bool isSetupWizard: false
    property bool isActive: main.state === "DiagnosticState" && diagnosticsMain.currentSelectedIndex >= 0 || root.isSetupWizard

    function deleteAllChannels() {
        tv.channelList.channels.clear()
        tv.channelList.doSave()
        selectMultipleCheckbox.value = false
        channelListBox.multipleSelectEnabled = selectMultipleCheckbox.value
    }

    Component.onDestruction: if (root.isActive) tv.on = false

    onIsActiveChanged: {
        if (isActive){
            if (channelListBox.channelListGrid.currentIndex >= 0)
                previewTv.previewChannel(channelListBox.channelListGrid.currentIndex)
        } else {
            tv.on = false
        }
    }

    ChannelListBox {
        id: channelListBox

        width: 0.7 * root.width
        height: 0.65 * root.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: -rightControlArea.width/2
        anchors.top: rightControlArea.top

        isSelectable: true

        onUpdatePreview: {
            if (selectedIndex >= 0){
                previewTimer.index = selectedIndex
                previewTimer.restart()
            }
        }
    }

    ModalDialog {
        id: deletePopup
        dismissable: true
        blockKeys: true

        Item {
            z: deletePopup.z + 1
            anchors.fill: parent

            TextLabel {
                id: warningText
                width: parent.width
                text: qsTr("DELETE CHANNEL?") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40 * main.vScaleFactor
            }

            Column {
                id: touchControls2
                width: 300 * main.hScaleFactor
                height: 200 * main.vScaleFactor
                spacing: 20 * main.vScaleFactor
                anchors {
                    top: warningText.bottom
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }

                StandardButton {
                    id: deleteBtn
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        tv.channelList.removeAChannel(channelListBox.channelListGrid.currentIndex)
                        tv.channelList.doSave()
                        if(tv.channelList.count === 0) {
                            selectMultipleCheckbox.value = false
                            channelListBox.multipleSelectEnabled = selectMultipleCheckbox.value
                        }

                        deletePopup.hide()
                    }
                    buttonText: qsTr("Delete Channel") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }

                StandardButton {
                    id: cancelBtn
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        deletePopup.dismissed()
                    }
                    buttonText: qsTr("Cancel") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }

                StandardButton {
                    id: deleteAllBtn
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        deletePopup.hide()
                        deleteAllPopup.show()
                    }
                    buttonText: qsTr("Delete ALL Channels") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    ModalDialog {
        id: deleteSelectedPopup
        dismissable: true
        blockKeys: true

        Item {
            z: parent.z + 1
            anchors.fill: parent

            TextLabel {
                id: warningText2
                width: parent.width
                text: qsTr("DELETE SELECTED CHANNELS?") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40 * main.vScaleFactor
            }

            Column {
                width: 300 * main.hScaleFactor
                height: 200 * main.vScaleFactor
                spacing: 20 * main.vScaleFactor
                anchors {
                    top: warningText2.bottom
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }

                StandardButton {
                    id: deleteBtn2
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        tv.channelList.removeSelectedChannels()
                        channelListBox.channelListGrid.currentIndex = -1
                        if(tv.channelList.count === 0) {
                            selectMultipleCheckbox.value = false
                            channelListBox.multipleSelectEnabled = selectMultipleCheckbox.value
                        }

                        deleteSelectedPopup.hide()
                    }
                    buttonText: qsTr("Delete Selected Channels") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }

                StandardButton {
                    id: cancelBtn2
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        deleteSelectedPopup.dismissed()
                    }
                    buttonText: qsTr("Cancel") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }

                StandardButton {
                    id: deleteAllBtn2
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        deleteSelectedPopup.hide()
                        deleteAllPopup.show()
                    }
                    buttonText: qsTr("Delete ALL Channels") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    Loader {
        id: addChannelLoader

        function loadItem() {
           sourceComponent = addChannelComponent
        }

        function destroyItem() {
            sourceComponent = null
        }

        onLoaded: addChannelLoader.item.show()

        Component {
            id: addChannelComponent
            AddChannelPopup {
                id: addChannelPopup
                onChannelAdded: {
                    console.log("channelAdded", norm, network, number, freq)
                    var item = {id: "A"+freq, number: number, freq: freq, name: network, enabled:"true", analog:true, norm: norm}
                    tv.channelList.addChannel(item)
                    root.doSave()
                    var idx = tv.channelList.findChannel(item.id)
                    if (idx >= 0){
                        channelListBox.channelListGrid.currentIndex = idx
                    }
                    hide()
                    addChannelLoader.destroyItem()
                }
                onDismissed: {
                    addChannelLoader.destroyItem()
                }
            }
        }
    }

    ModalDialog {
        id: deleteAllPopup
        dismissable: true
        blockKeys: true

        Item {
            z: parent.z + 1
            anchors.fill: parent

            TextLabel {
                id: warningText3
                width: parent.width
                text: qsTr("DELETE ALL?") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40 * main.vScaleFactor
            }

            Column {
                id: touchControls3
                width: 300 * main.hScaleFactor
                height: 200 * main.vScaleFactor
                spacing: 20 * main.vScaleFactor
                anchors {
                    top: warningText3.bottom
                    topMargin: 20 * main.vScaleFactor
                    horizontalCenter: parent.horizontalCenter
                }

                StandardButton {
                    id: deleteAllBtn3
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        root.deleteAllChannels()
                        deleteAllPopup.hide()
                    }
                    buttonText: qsTr("Delete ALL Channels") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }

                StandardButton {
                    id: cancelBtn3
                    width: 0.9 * parent.width
                    height: 50 * main.vScaleFactor

                    anchors.horizontalCenter: parent.horizontalCenter

                    onClicked: {
                        deleteAllPopup.dismissed()
                    }
                    buttonText: qsTr("Cancel") + application.emptyString
                    label.font.pixelSize: 18 * main.vScaleFactor
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    Timer {
        id: previewTimer
        property int index: 0

        interval: 500
        running: false
        repeat: false
        onTriggered: {
            previewTv.previewChannel(previewTimer.index)
        }
    }

    Rectangle {
        id: rightControlArea

        width: 0.25 * root.width
        height: channelListBox.height
        color: "#7f8082"
        anchors { left: channelListBox.right; top: parent.top; topMargin: 80 * main.vScaleFactor }

        VLCSurface{
            id: previewTv
            player: tv.player
            aspect: tv.aspect
            playing: channelListBox.channelListGrid.currentIndex >= 0 && root.isActive
            width: 0.9 * parent.width
            height: width/tv.aspect
            anchors { top: parent.top; right: parent.right; margins: 0.05 * parent.width }
            anchors.topMargin: 0.05 * parent.width
            onPlayingChanged: tv.on = playing
            z: 1000

            function previewChannel(index) {
                var mrl = tv.channelList.mrl(index)
                tv.on = true
                tv.mrl = mrl
                tv.currentChannel= index
            }
        }

        Column {
            id: touchControls

            anchors {
                top: rightControlArea.top;
                topMargin: 200 * main.vScaleFactor
                left: rightControlArea.left
                leftMargin: 0.05 * width
            }
            width: rightControlArea.width
            height: rightControlArea.height
            spacing: 20 * main.vScaleFactor
            z: 1000//parent.z + 1

            StandardButton {
                id: deleteChannelButton
                visible: tv.channelList.channels.count > 0
                buttonText: qsTr("Delete Channel") + application.emptyString
                width: 0.9 * parent.width
                height: 51 * main.vScaleFactor
                label.font { pixelSize: 12 * main.vScaleFactor }
                onClicked: {
                    deletePopup.show()
                }
            }

            StandardButton {
                id: addChannelButton
                visible: !options.iptvEnabled
                buttonText: qsTr("Add Analog Channel") + application.emptyString
                width: 0.9 * parent.width
                height: 51 * main.vScaleFactor
                label.font { pixelSize: 12 * main.vScaleFactor }
                onClicked: {
                    addChannelLoader.loadItem()
                }
            }

            StandardButton {
                id: iptvAddButton
                visible: options.iptvEnabled
                buttonText: qsTr("Add Channel") + application.emptyString
                width: 0.9 * parent.width
                height: 51 * main.vScaleFactor
                label.font { pixelSize: 12 * main.vScaleFactor }
                onClicked: {
                }
            }

            Checkbox {
                id: selectMultipleCheckbox
                visible: tv.channelList.channels.count > 0
                label: qsTr("Select Multiple Channels")
                width: 0.9 * parent.width
                height: 30 * main.vScaleFactor
                font.pixelSize: 12 * main.vScaleFactor
                onValueChanged: channelListBox.multipleSelectEnabled = value
            }

            StandardButton {
                id: deleteSelectedBtn
                width: 0.9 * parent.width
                height: 70* main.vScaleFactor
                visible: tv.channelList.channels.count > 0 && channelListBox.multipleSelectEnabled

                buttonText: qsTr("Delete Selected Channels") + application.emptyString
                label.font.pixelSize: 12 * main.vScaleFactor

                onClicked: {
                    deleteSelectedPopup.show()
                }
            }
        }
    }

    // When scan is complete, show how many services were found.
    Text {
        id: channelsFoundText
        text: {
            if (channelListBox.channelListGrid.count > 0)
                return qsTr("Number of Channels:") + " " + channelListBox.channelListGrid.count + application.emptyString
            else
                return qsTr("No Channels Configured") + application.emptyString
        }
        anchors.bottom: channelListBox.top
        anchors.bottomMargin: 15 * main.vScaleFactor
        anchors.left: channelListBox.left
        font { pixelSize: 18 * main.vScaleFactor; bold: true }
    }

    FileIO {
        id: fileio
    }
}
