
import QtQuick 2.4
import Process 0.1
import "."
import "../Television"
import "../"
import FileIO 1.0
import Components 1.0

Item {
    id: root
    property QtObject filteredCountryList
    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property string tuner_id: tv.tunerType
//    property string tuner_id: "DVB"
    property variant scanner: dvbscan
    property bool isSetupWizard: false
    property bool isActive: (main.state === "DiagnosticState" && diagnosticsMain.currentSelectedIndex >= 0) || root.isSetupWizard
    property string cursorState: "setup" // setup or next

    anchors.fill: parent

    function doSave() {
        // take the channels in scanChannelList and dump them to tv.channelList
        tv.channelList.channels.clear()
        for(var i=0 ; i < scanChannelList.channels.count ; i++) {
            var item = scanChannelList.channels.get(i)
            var words = String(item.name).split(" ")
            if(words[0] === "service_id"){
                //console.log("channel name=" + item.name)
                //console.log("service string" + words[1])
                words[0]=qsTr("Service ")
                item.name=words.join(" ")
                //console.log("new name is:" + item.name)
            }

            tv.channelList.addChannel(item)
        }
        tv.channelList.saveChannels(tv.channelFile)
        tv.channelList.readChannels(tv.channelFile)
        syncDisk.start()
    }

    function startScan() {
        if(digitalCheckbox.value) {
            var tunerType = root.getTunerType(countryListPopup.selectedData.tuner, cableAntToggle.value)
            dvbScanStart(countryListPopup.selectedData.code, tunerType, cableAntToggle.value)
            // digital will fire analog after it completes, if it also checked
        }
        else if(analogCheckbox.value) {
            analogScanStart(countryListPopup.selectedData.code, cableAntToggle.value, analogSelector.selectedData.value)
        }
        scanningDialog.show()
    }

    Component.onCompleted: {
        if(!tuner_id) {
            print("no tuner installed")
            noTuner.showWithTimeout(10)
        }
        print("tuner type:" + tuner_id)

        if(tuner_id == "DVB") {
            cableAntToggle.value = false
            print("tuner type is " + tuner_id + ".  Hiding cable option.")
        }

        // get current channel list from TV element
        for(var i=0 ; i < tv.channelList.channels.count ; i++) {
            var item = tv.channelList.channels.get(i)
            scanChannelList.addChannel(item)
        }
    }

    onTuner_idChanged: {
        filterCountries(tuner_id)
        // force re-selection of analog standard
        countryListPopup.currentIndexChanged()
    }

    // We are using a universal tuner, so compute a string representing the
    // "tuner type" used and store it
    function getTunerType(tunerType, isCable) {
        if (root.tuner_id === "DVB-T2") {
            return isCable ? "DVB-C" : "DVB-T2"
        }

        if (tunerType === "ATSC") {
            return isCable ? "QAM" : "ATSC"
        } else if (tunerType === "DVB") {
            return isCable ? "DVB-C" : "DVB-T"
        } else if (tunerType === "ISDB") {
            return "ISDB-T"
        } else if (tunerType === "DMB") {
            return "DTMB"
        }

        return "ATSC"
    }

    function dvbScanStart(countryCode, tunerType, isCable) {
        tv.player.stop()
        if (root.tuner_id === "DVB-C") {
            // force frontend mode switch for multi-standard tuner
            setDelsys.arguments = ["/dev/dvb/adapter0/frontend0", isCable ? "1" : "3"]
            setDelsys.start()
            setDelsys.waitForFinished(5000);
        }

        // Scan by network ID
        if (isCable && addNetworkDialog.networkId !== "00000") {
            dvbscan.initialScanData = "C " + (addNetworkDialog.frequency * 1000) + " " + addNetworkDialog.symbolRate * 1000 + " AUTO " + addNetworkDialog.modulation
            dvbscan.networkId = addNetworkDialog.networkId
        } else {
            dvbscan.initialScanData = ""
            dvbscan.networkId = 0
        }

        root.scanner = dvbscan
        dvbscan.tunerType = tunerType
        dvbscan.startScan(countryCode, isCable, inversionCheckbox.value)
    }

    function analogScanStart(countryCode, isCable, norm) {
        tv.stop()
        root.scanner = analogScanner
        analogScanner.startScan(countryCode, isCable, norm)
    }

    function stopScan() {
        root.scanner.stopScan()
        scanningDialog.hide()
    }

    function filterCountries(tuner) {
        if (tuner === "DVB-C" || tuner === "DVB-T2") {
            tuner = "DVB"
        } else if (tuner === "H1265") { //H1265 pcie tuner is North American only.
            tuner="ATSC"
        }

        var newModel = tv_country_codes
        if (tuner === "MGT") {
            // No filtering for MGT
        } else if (tuner === "H975") {
            // Show all except DMB and ISDB for H975
            for( var i = tv_country_codes.count-1 ; i >= 0 ; i--) {
                if ( newModel.get(i).tuner == "DMB" || newModel.get(i).tuner == "ISDB") {
                    newModel.remove(i)
                }
            }
        } else {
            for( var i = tv_country_codes.count-1 ; i >= 0 ; i--) {
                if ( newModel.get(i).tuner != tuner ) {
                    newModel.remove(i)
                }
            }
        }
        root.filteredCountryList = newModel
        countryListPopup.model = newModel
    }

    Process {
        id: setDelsys
        program: "scripts/set_delsys"
        onReadyReadStandardError: {
            var outp = String(setDelsys.readAllStandardError())
            console.log("Output from setDelsys:", outp)
        }
    }

    TextLabel {
        id: title
        text:  qsTr("TV SETUP")
        color: main.colorFontDark
        anchors.top: parent.top
        anchors.topMargin: 20 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        font.pixelSize: 24 * main.vScaleFactor
    }

    Checkbox {
        id: iptvCheckbox
        label: "IPTV"

        anchors {
            left: channelListBox.left
            bottom: channelsFoundText.top
            bottomMargin: 15* main.vScaleFactor
        }
        width: setupScanDialog.centerWidth

        value: options.iptvEnabled
        onPressed: options.iptvEnabled = value
        Component.onCompleted: value = options.iptvEnabled
    }

    ChannelListBox {
        id: channelListBox

        width: 0.7 * root.width
        height: 0.65 * root.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: -rightControlArea.width/2
        anchors.top: rightControlArea.top
    }

    Text {
        id: instructionalText
        text: qsTr("NOTE: If you are having trouble receiving channels after scanning, or having issues with channel clarity after confirming good connections to this machine, please contact your TV provider to ensure proper scan settings and signal strength.") + application.emptyString
        wrapMode: Text.WordWrap
        anchors.left: channelListBox.left
        anchors.right: rightControlArea.right
        anchors.top: channelListBox.bottom
        anchors.topMargin: 5 * main.vScaleFactor
        font { pixelSize: 17 * main.vScaleFactor; bold: true }
    }

    Rectangle {
        id: rightControlArea

        width: 0.25 * root.width
        height: channelListBox.height
        color: "#7f8082"
        anchors { left: channelListBox.right; top: title.bottom; topMargin: 60 * main.vScaleFactor }
    }

    MessagePopup {
        id: noTuner
        message: qsTr("No Tuner Installed.") + application.emptyString
        dismissable: true
    }

    Connections {
        target: root.scanner
        onScanComplete: {
            print("channel scanning complete.")
            for(var i=0 ; i < root.scanner.channels.count ; i++)
                scanChannelList.addChannel(root.scanner.channels.get(i), true)

            if (root.scanner.scanType === "digital" && analogCheckbox.value && !root.scanner.aborted) {
                analogScanStart(countryListPopup.selectedData.code, cableAntToggle.value, analogSelector.selectedData.value)
            } else {
                scanningDialog.hide()
            }

            root.doSave()
        }
    }

    Column {
        id: controlList

        anchors {
            top: rightControlArea.top;
            topMargin: spacing
            left: rightControlArea.left
            leftMargin: 0.05 * width
        }
        width: rightControlArea.width
        height: rightControlArea.height
        spacing: 20 * main.vScaleFactor
        z: 100

        StandardButton {
            id: importButton

            enabled: root.isUsbDeviceLoaded && fileio.exists("/media/usb/tv_conf.xml")
            buttonText: qsTr("Import Scan from USB") + application.emptyString
            width: 0.9 * parent.width
            height: 51 * main.vScaleFactor
            label.font.pixelSize: 12 * main.vScaleFactor

            onClicked: {
                scanChannelList.channels.clear()

                // this call is asynchronous - channels are added in the onLoadComplete callback
                externalChanList.readChannels("/media/usb/tv_conf.xml")
            }
        }

        StandardButton {
            id: exportButton

            enabled: scanChannelList.channels.count > 0 && root.isUsbDeviceLoaded
            buttonText: qsTr("Export Scan to USB") + application.emptyString
            width: 0.9 * parent.width
            height: 51 * main.vScaleFactor
            label.font { pixelSize: 12 * main.vScaleFactor }

            onClicked: {
                scanChannelList.saveChannels("/media/usb/tv_conf.xml")

                exportSuccess.showWithTimeout(1)
            }
        }

        StandardButton {
            id: startScan

            enabled: digitalCheckbox.value || analogCheckbox.value
            buttonText: qsTr("Begin Channel Scan") + application.emptyString
            width: 0.9 * parent.width
            height: 51 * main.vScaleFactor
            label.font { pixelSize: 12 * main.vScaleFactor }

            onClicked: {
                setupScanDialog.show()
            }
        }
    }

    // When scan is complete, show how many services were found.
    Text {
        id: channelsFoundText
        text: {
            if (root.scanner.scanning)
                return ""
            else if (scanChannelList.channels.count > 0)
                return qsTr("Number of Channels:") + " " + scanChannelList.channels.count + application.emptyString
            else
                return qsTr("No Channels Configured") + application.emptyString
        }
        anchors.bottom: channelListBox.top
        anchors.bottomMargin: 7 * main.vScaleFactor
        anchors.left: channelListBox.left
        font { pixelSize: 18* main.vScaleFactor; bold: true }
    }

    MessagePopup {
        id: exportNetworkMessage
        message: qsTr("Exported to Network") + application.emptyString
        dismissable: false
    }

    AddNetworkDialog {
        id: addNetworkDialog
        onDismissed: {
            setupScanDialog.show()
        }

    }

    ModalDialog {
        id: setupScanDialog
        dismissable: true
        dismissAnywhere: false
        opacity: 1
        width: main.widthDialog
        height: 0.75*root.height
        anchors.centerIn: parent
        z: 200

        property int centerWidth: width/2

        onVisibleChanged: {
            if (!visible)
                countryListPopup.close()
        }

        // close button
        CloseButton {
            id: btExitButton

            z: 200

            anchors {
                right: parent.right
                rightMargin: 40 * main.hScaleFactor
                top: parent.top
                topMargin: 40 * main.vScaleFactor
            }

            onClicked: {
                countryListPopup.close()
                setupScanDialog.hide()
            }
        }

        Column {
            id: tvSetupControlList
            width: setupScanDialog.centerWidth
            height: parent.height
            anchors.top: parent.top
            anchors.topMargin: 15 * main.vScaleFactor
            anchors.horizontalCenter: parent.horizontalCenter
            spacing: 10 * main.vScaleFactor
            z: parent.z + 1

            onVisibleChanged: {
                if (!visible)
                    countryListPopup.close()
            }

            TextLabel {
                id: scanHeader
                width: setupScanDialog.centerWidth
                text: qsTr("CHANNEL SCAN") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter

                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 30 * main.vScaleFactor
            }

            Checkbox {
                id: analogCheckbox
                label: "ANALOG"
                labelColor: main.colorFontLight
                edgeColor: main.colorFontLight

                width: setupScanDialog.centerWidth

                value: false
            }

            Checkbox {
                id: digitalCheckbox
                label: "DIGITAL"
                labelColor: main.colorFontLight
                edgeColor: main.colorFontLight

                width: setupScanDialog.centerWidth

                value: true
            }

            Item {
                id: normSelection
                height: visible ? 71 * main.vScaleFactor : 0
                width: 0.8*parent.width
                visible: analogCheckbox.value
                z:300

                ListModel {
                    id: analogCodes

                    ListElement { label: "NTSC-M"; value: "NTSC-M" }
                    ListElement { label: "NTSC"; value: "NTSC" }
                    ListElement { label: "PAL-BG"; value: "PAL-BG" }
                    ListElement { label: "PAL-DK"; value: "PAL-DK" }
                    ListElement { label: "PAL-I"; value: "PAL-I" }
                    ListElement { label: "PAL-N"; value: "PAL-N" }
                    ListElement { label: "SECAM-L"; value: "SECAM-L" }
                    ListElement { label: "SECAM-B"; value: "SECAM-B" }
                    ListElement { label: "SECAM-DK"; value: "SECAM-DK" }
                }

                ComboBox {
                    id: analogSelector

                    comboHeight: 40 * main.vScaleFactor
                    itemHeight: 35 * main.vScaleFactor
                    width: parent.width; height: comboHeight
                    model: analogCodes
                    anchors.verticalCenter: parent.verticalCenter
                }
            }

            ToggleButton{
                id: cableAntToggle

                width: setupScanDialog.centerWidth
                height: 30 * main.vScaleFactor
                pos0label: qsTr("Antenna") + application.emptyString
                pos1label: qsTr("Cable") + application.emptyString
                labelcolor: main.colorFontLight
                highlightColor: main.colorFontLight
                posFont.pixelSize: 20* main.vScaleFactor
                value: true // Set default to Cable
                enabled: root.tuner_id !== "DVB" || !digitalCheckbox.value

                Connections {
                    target: digitalCheckbox
                    onValueChanged: if(digitalCheckbox.value && (tuner_id == "DVB")) {
                                        cableAntToggle.value = false
                                    }
                }
            }

            Row {
                id: networkIdSelector
                property bool selected: false
                // Only show for DVB-C cable
                visible: digitalCheckbox.value
                         && root.getTunerType(countryListPopup.selectedData === undefined ? "ATSC" : countryListPopup.selectedData.tuner, cableAntToggle.value) === "DVB-C"
                spacing: 10 * main.vScaleFactor
                height: 40 * main.vScaleFactor
                width: setupScanDialog.centerWidth

                TextLabel {
                    id: networkIDLabel
                    text: qsTr("NETWORK ID") + application.emptyString
                    font.pixelSize: 20 * main.vScaleFactor
                    anchors.verticalCenter: parent.verticalCenter
                }

                Rectangle {
                    id: netIdRect
                    color: main.colorHighlightBackground
                    height: 30 * main.vScaleFactor
                    width: parent.width - networkIDLabel.width - parent.spacing

                    TextLabel {
                        anchors.centerIn: parent
                        text: addNetworkDialog.networkId !== "00000" ? addNetworkDialog.networkId : qsTr("Auto") + application.emptyString
                    }
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            setupScanDialog.hide()
                            addNetworkDialog.show()
                            buzzer.buttonPress()
                        }
                        Rectangle {
                            anchors.fill: parent
                            color: main.colorGenericStimulusArea
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }
            }

            Item {
                id: countrySelect
                width: setupScanDialog.centerWidth
                height: 71 * main.vScaleFactor

                Rectangle {
                    color: main.colorHighlightBackground
                    width: parent.width
                    height: 40 * main.vScaleFactor
                    anchors.verticalCenter: parent.verticalCenter

                    Text {
                        id: countryValue
                        text: countryListPopup.selectedData ? qsTranslate("TelevisionRegionList", countryListPopup.selectedData.label) + application.emptyString : ""
                        anchors.left: parent.left
                        anchors.leftMargin: 15 * main.hScaleFactor
                        anchors.verticalCenter: parent.verticalCenter
                        font.bold: false
                        color: main.colorFontLight
                    }

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            countryListPopup.visible = true
                            buzzer.buttonPress()
                        }
                        Rectangle {
                            anchors.fill: parent
                            color: main.colorGenericStimulusArea
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }

            }

            Checkbox {
                id: inversionCheckbox
                label: qsTr("Use Inversion") + application.emptyString
                labelColor: main.colorFontLight
                edgeColor: main.colorFontLight

                width: setupScanDialog.centerWidth

                value: false
            }

            Item {
                height: 10 * main.vScaleFactor
                width: setupScanDialog.centerWidth
            }

            StandardButton {
                id: beginScanButton

                enabled: digitalCheckbox.value || analogCheckbox.value
                buttonText: qsTr("BEGIN CHANNEL SCAN") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter

                onClicked: {
                    setupScanDialog.hide()
                    if (scanChannelList.channels.count > 0) {
                        clearChannelDialog.show()
                    } else {
                        setupScanDialog.hide()
                        root.startScan()
                    }
                }
            }
        }

        CountryListPopup {
            id: countryListPopup
            model: filteredCountryList
            modelTranslationContext: "TelevisionRegionList"
            anchors {
                right: parent.right
                top: parent.top
                topMargin: -20 * main.vScaleFactor
            }
            onCurrentIndexChanged: {
                // autoselect analog norm matching the selected country
                var val = selectedData.norm
                for (var i = 0; i < analogCodes.count; i++) {
                    if (analogCodes.get(i).value === val) {
                        analogSelector.currentIndex = i
                        break
                    }
                }
            }
        }

        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    ConfirmationPrompt {
        id: clearChannelDialog
        prompt: qsTr("Clear existing channel list?") + application.emptyString
        visible: false
        z: setupScanDialog.z + 1
        onNoClicked: {
            clearChannelDialog.hide()
            root.startScan()
        }
        onYesClicked: {
            scanChannelList.channels.clear()
            root.doSave()
            clearChannelDialog.hide()
            root.startScan()
        }
    }

    ModalDialog {
        id: scanningDialog
        dismissable: true

        onDismissed: {
            root.stopScan()
        }

        Column {
            z: scanningDialog.z + 1
            anchors.fill: parent

            spacing: 20 * main.vScaleFactor

            TextLabel {
                id: warningText
                width: parent.width
                height: 49 * main.vScaleFactor
                text: qsTr("Scanning in progress") + application.emptyString
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40* main.vScaleFactor
            }

            ProgressBar {
                visible: root.scanner.scanning
                width: 0.5 * parent.width
                height: 40 * main.vScaleFactor
                value: root.scanner.progress * 100
                anchors.horizontalCenter: parent.horizontalCenter
                showLabel: true
            }

            Text {
                id: scanTypeText
                anchors.horizontalCenter: parent.horizontalCenter
//                visible: root.scanner.scanning
                color: main.colorFontLight
                text: qsTr("Scanning ") + (root.scanner.scanType === "digital" ? qsTr("digital") : qsTr("analog")) + "..." + application.emptyString
            }

            // while scanning show how many frequencies are tuned
            Text {
                id: frequenciesFoundText
//                visible: root.scanner.scanning
                anchors.horizontalCenter: parent.horizontalCenter
                color: main.colorFontLight
                text: qsTr("Frequencies Found: ") + (dvbscan.signalsFound + analogScanner.signalsFound) + application.emptyString
            }

            Text {
                id: currentScanText
                anchors.horizontalCenter: parent.horizontalCenter
                visible: !root.scanner.networkId
                color: main.colorFontLight
                text: qsTr("Scanning Frequency: ") + root.scanner.scanFreq + application.emptyString
            }

            TransientText {
                id: foundChannelText
                anchors.horizontalCenter: parent.horizontalCenter
                color: main.colorFontLight
                Connections {
                    target: root.scanner
                    onChannelFound: {
                        if (root.scanner.networkId > 0)
                            return

                        if (chandata.name) {
                            foundChannelText.setText(qsTr("Found ") + chandata.name)
                        }
                    }
                }
            }

            StandardButton {
                id: stopScanButton
                height: 80 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                buttonText: qsTr("Stop Scan") + application.emptyString

                onClicked: {
                    root.stopScan()
                }
            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    TelevisionRegionList {
        id: tv_country_codes
    }

    TelevisionScanDVB {
        id: dvbscan
        tunerType: "ATSC"
//        useMock: true
    }

    TelevisionScanAnalog {
        id: analogScanner
//        useMock: true
    }

    MessagePopup {
        id: importSuccess
        message: qsTr("Channel List Loaded") + application.emptyString
        dismissable: false
    }

    MessagePopup {
        id: exportSuccess
        message: qsTr("Channel List Saved to Device") + application.emptyString
        dismissable: false
    }

    ChannelList {
        id: externalChanList

        onLoadComplete: {
            if (externalChanList.channels.count > 0) {
                for (var i = 0; i < externalChanList.channels.count; i++) {
                    var item = externalChanList.channels.get(i)
                    scanChannelList.addChannel(item)
                }

                importSuccess.showWithTimeout(1)
                root.doSave() //Save channels after importing
            }
        }
    }

    ChannelList {
        id: scanChannelList
    }

    FileIO {
        id: fileio
    }

}
