import QtQuick 2.4
import Qt.labs.settings 1.0

Settings {
    id: root
    property Item defaults: Item {
        id: defaults
        property string defaultLanguageDefault: "en"
        property bool metricUnitsDefault: false
        property int cooldownTimeDefault: 300 // 5 minutes
        property int workoutTimeMaxDefault: 60 // 60 minutes
        property bool saveWorkoutEnabledDefault: false
        property int workoutSetupTimeoutDefault: 45
        property int workoutMenuTimeoutDefault: 45
        property int workoutSummaryTimeoutDefault: 60
        property int screenSaveTimeoutDefault: 600 // 10 minutes
        property int sleepModeTimeoutDefault: 0 // 0 minutes
        property int dimDisplayTimeoutDefault: 600 // 10 minutes
        property bool contactHRPriorityDefault: true
        property bool simulationModeDefault: false
        property bool showStimulusAreaDefault: false
        property bool browserEnabledDefault: false
        property bool disableNoPedalStopDefault: false
        property bool showCursorDefault: false
        property int lcdBrightnessDefault: 80
        property bool ignoreFaultsDefault: false
        property int resumeWorkoutTimeoutDefault:60 // 60 seconds
        property bool erpEnabledDefault: false
        property int erpTimeoutDefault: 1800 // 30 minutes
        property bool disableBuzzerDefault: true
        property string homeURLDefault: "http://www.truefitness.com"
        property string browserCountryCodeDefault: "us"
        property bool cabEnabledDefault: false
        property string cabSettingDefault: "STB_00"
        property bool en957EnabledDefault: false
        property int autoChannelChangeIntervalDefault: 0
        property int triggerUSBFaultIntervalDefault: 0
        property bool triggerWorkoutAutoRestartDefault: false
        property bool ppdEnabledDefault: true
        property int ppdTimer1Default: 60 //ppd timer 1 60 seconds
        property int ppdTimer2Default: 20 // ppd timer 2 20 seconds
        property int resumeWorkoutTimeoutDefault_ES900:0
        property int sleepModeTimeoutDefault_ES900: 1800 // 30 minutes
        property bool saveWorkoutEnabledDefault_ES900: true
        property int workoutTimeMaxDefault_ES900: 99 // 99 minutes
        property bool tvNoFrameDropDefault: false
        property bool tvHwAccelDefault: false
        property bool loggingEnabledDefault: false
        property bool iptvEnabledDefault: false
        property bool modestyModeDefault: false
        property bool maxSpeedRegulatorEnabledDefault: false
        property int maxSpeedRegulatorValueDefault: 0
        property bool diagnosticsUploadEnabledDefault: true
        property bool manufacturingTestModeEnabledDefault: false
        property bool setupWizardCompletedDefault: false
        property bool virtualVideoSpeedControlledPlaybackRate: false
    }

    property string defaultLanguage: defaults.defaultLanguageDefault
    property bool metricUnits: defaults.metricUnitsDefault
    property int cooldownTime: defaults.cooldownTimeDefault
    property int workoutTimeMax: defaults.workoutTimeMaxDefault
    property bool saveWorkoutEnabled: defaults.saveWorkoutEnabledDefault
    property int workoutSetupTimeout: defaults.workoutSetupTimeoutDefault
    property int workoutMenuTimeout: defaults.workoutMenuTimeoutDefault
    property int workoutSummaryTimeout: defaults.workoutSummaryTimeoutDefault
    property int screenSaveTimeout: defaults.screenSaveTimeoutDefault
    property int dimDisplayTimeout: defaults.dimDisplayTimeoutDefault
    property int sleepModeTimeout: defaults.sleepModeTimeoutDefault
    property bool contactHRPriority: defaults.contactHRPriorityDefault
    property bool simulationMode: defaults.simulationModeDefault
    property bool showStimulusArea: defaults.showStimulusAreaDefault
    property bool browserEnabled: defaults.browserEnabledDefault
    property bool disableNoPedalStop: defaults.disableNoPedalStopDefault
    property int lcdBrightness: defaults.lcdBrightnessDefault
    property bool showCursor: defaults.showCursorDefault
    property bool ignoreFaults: defaults.ignoreFaultsDefault
    property int resumeWorkoutTimeout: defaults.resumeWorkoutTimeoutDefault
    property bool disableBuzzer: defaults.disableBuzzerDefault
    property bool ppdEnabled: defaults.ppdEnabledDefault
    property int personPresentTimeout: defaults.ppdTimer1Default
    property int noUserDetectedDialogTimeout: defaults.ppdTimer2Default
    property bool tvNoFrameDrop: defaults.tvNoFrameDropDefault
    property bool tvHwAccel: defaults.tvHwAccelDefault
    property bool loggingEnabled: defaults.loggingEnabledDefault
    property int autoChannelChangeInterval: defaults.autoChannelChangeIntervalDefault
    property int triggerUSBFaultInterval: defaults.triggerUSBFaultIntervalDefault
    property bool triggerWorkoutAutoRestart: defaults.triggerWorkoutAutoRestartDefault
    property bool iptvEnabled: defaults.iptvEnabledDefault
    property bool erpEnabled: nucleus.erpIsSupported
    property int erpTimeout: defaults.erpTimeoutDefault
    property string homeURL: defaults.homeURLDefault
    property string browserCountryCode: defaults.browserCountryCodeDefault
    property bool cabEnabled: defaults.cabEnabledDefault
    property string cabSetting: defaults.cabSettingDefault
    property bool modestyMode: defaults.modestyModeDefault
    property bool en957Enabled: defaults.en957EnabledDefault
    property bool maxSpeedRegulatorEnabled: defaults.maxSpeedRegulatorEnabledDefault
    property int maxSpeedRegulatorValue: defaults.maxSpeedRegulatorEnabledDefault
    property bool diagnosticsUploadEnabled: defaults.diagnosticsUploadEnabledDefault
    property bool manufacturingTestModeEnabled: defaults.manufacturingTestModeEnabledDefault
    property bool setupWizardCompleted: defaults.setupWizardCompletedDefault
    property bool virtualVideoSpeedControlledPlaybackRate: defaults.virtualVideoSpeedControlledPlaybackRate
    property string modelString: machineinfo.modelString

    function resetToDefaults() {
        defaultLanguage = defaults.defaultLanguageDefault
        metricUnits = defaults.metricUnitsDefault
        cooldownTime = defaults.cooldownTimeDefault
        workoutSetupTimeout = defaults.workoutSetupTimeoutDefault
        workoutMenuTimeout = defaults.workoutMenuTimeoutDefault
        workoutSummaryTimeout = defaults.workoutSummaryTimeoutDefault
        screenSaveTimeout = defaults.screenSaveTimeoutDefault
        dimDisplayTimeout = defaults.dimDisplayTimeoutDefault
        contactHRPriority = defaults.contactHRPriorityDefault
        simulationMode = defaults.simulationModeDefault
        showStimulusArea = defaults.showStimulusAreaDefault
        browserEnabled = defaults.browserEnabledDefault
        disableNoPedalStop = defaults.disableNoPedalStopDefault
        lcdBrightness = defaults.lcdBrightnessDefault
        showCursor = defaults.showCursorDefault
        ignoreFaults = defaults.ignoreFaultsDefault
        disableBuzzer = defaults.disableBuzzerDefault
        ppdEnabled = defaults.ppdEnabledDefault
        personPresentTimeout = defaults.ppdTimer1Default
        noUserDetectedDialogTimeout = defaults.ppdTimer2Default
        tvNoFrameDrop = defaults.tvNoFrameDropDefault
        tvHwAccel = defaults.tvHwAccelDefault
        loggingEnabled = defaults.loggingEnabledDefault
        autoChannelChangeInterval = defaults.autoChannelChangeIntervalDefault
        triggerUSBFaultInterval = defaults.triggerUSBFaultIntervalDefault
        triggerWorkoutAutoRestart = defaults.triggerWorkoutAutoRestartDefault
        iptvEnabled = defaults.iptvEnabledDefault
        erpTimeout = defaults.erpTimeoutDefault
        erpEnabled = defaults.erpEnabledDefault
        nucleus.setERPSupport(erpEnabled) // Keep nucleus ERP consistent with default value
        homeURL = defaults.homeURLDefault
        browserCountryCode = defaults.browserCountryCodeDefault
        cabEnabled = defaults.cabEnabledDefault
        cabSetting = defaults.cabSettingDefault
        modestyMode = defaults.modestyModeDefault
        en957Enabled = defaults.en957EnabledDefault
        maxSpeedRegulatorEnabled = defaults.maxSpeedRegulatorEnabledDefault
        maxSpeedRegulatorValue = defaults.maxSpeedRegulatorValueDefault
        diagnosticsUploadEnabled = defaults.diagnosticsUploadEnabledDefault
        manufacturingTestModeEnabled = defaults.manufacturingTestModeEnabledDefault
        setupWizardCompleted = defaults.setupWizardCompletedDefault
        virtualVideoSpeedControlledPlaybackRate = defaults.virtualVideoSpeedControlledPlaybackRate
        setProductDependentDefaults(machineinfo.modelString)
    }

    function setProductDependentDefaults(modelstring){
        // Forcefully set defaults when a product is selected...
        if(modelstring === "TES900" || modelstring === "TE900"){
            resumeWorkoutTimeout = defaults.resumeWorkoutTimeoutDefault_ES900
            sleepModeTimeout = defaults.sleepModeTimeoutDefault_ES900
            saveWorkoutEnabled = defaults.saveWorkoutEnabledDefault_ES900
            workoutTimeMax = defaults.workoutTimeMaxDefault_ES900
        } else {
            resumeWorkoutTimeout = defaults.resumeWorkoutTimeoutDefault
            sleepModeTimeout = defaults.sleepModeTimeoutDefault
            saveWorkoutEnabled = defaults.saveWorkoutEnabledDefault
            workoutTimeMax = defaults.workoutTimeMaxDefault
        }
    }
}

