// import QtQuick 2.4 // to target S60 5th Edition or Maemo 5
import QtQuick 2.4
import Components 1.0
import "../"
import WorkoutManager 1.0

Item {
    id: root

    property int elapsedTime: aWorkoutRunData.workoutTime + aWorkoutRunData.coolDownTimeproperty
    property bool isCustomIncline: (workoutProgram.programIndex == WorkoutProgram.TREAD_CUSTOM_HILLS_PROG || workoutProgram.programIndex == WorkoutProgram.CUSTOM_HILLS_PROG)
    property bool isCustomSpeed: (workoutProgram.programIndex == WorkoutProgram.TREAD_CUSTOM_SPEED_PROG || workoutProgram.programIndex == WorkoutProgram.CUSTOM_PACE_PROG)
    property string segmentColorSpeed: graphDisplayMode == 2 ? segmentColorMain : segmentColorSecondary
    property string segmentColorIncline: segmentColorMain

    property int graphDisplayMode: findGraphType(); // 0 = Incline Only; 1 = Both; 2 = Speed Only
    property string segmentColorMain: main.colorUIBorder
    property string segmentColorSecondary: main.colorHighlightBackground

    property variant model1
    property variant model2
    property int activeModel: 1
    property int columnWidth: 10 // Must not be below 10
    property int segmentFill: 0 // 0 - 100%
    property alias activeSegment: segmentView.currentIndex
    property int legendSize: 0.5*columnWidth
    property int legendFontSize: 12* main.vScaleFactor
    property int baseOffset: 0

    // set height of bar to draw based on the level or magnitude property of the profile segment, as appropriate
    property int borderWidth: 2 * main.scaleFactor
    property real yPositionOffsetPercent: 0.05 // Adds additional space between bottom of bar and bottom of container, if needed
    property int yPosition: (0.1 + yPositionOffsetPercent)*root.height
    property int topYPosition: 0.95*root.height
    property int maxBarHeight: root.topYPosition - root.yPosition // - root.borderWidth + heightDifferential - baseOffset
    property int axisPosition: {
        main.isStepmill ?
                (root.model2.maxLevel*root.yPosition)/root.model2.maxLevel  :
                (root.numNegativeValues*root.topYPosition+root.model1.maxLevel*root.yPosition)/(root.numNegativeValues+root.model1.maxLevel)
    }
    property int minBarHeight: 5 * main.vScaleFactor

    property bool hasNegativeValues: machineinfo.treadmillInclineMinPercentGrade < 0.0
    property int numNegativeValues: hasNegativeValues ? machineinfo.inclineTable.indexOf(0.0) : 0

    // Editable bars Only
    property bool editable: false
    property int editSegment
    property int editModel
    property bool active: false
    property bool useAlternativeHeight: false
    property real alternativeHeight
    property bool labelVisibility: editable
    property string segmentColorSpeedEdit: "#E62C2C" // Dark Pink
    property string segmentColorInclineEdit: "#C61D1D" // Dark Red
    property int currentSegmentX: segmentView.currentItem ? segmentView.x + segmentView.currentItem.x - segmentView.contentX : 0

    // Custom Summary Only
    property bool isSummary: false
//    property bool showLevel: false
//    property bool hasNegativeValues: false
//    property int numNegativeValues: 0

    clip: true
    onActiveSegmentChanged: editSegment = activeSegment
    onActiveModelChanged: editModel = activeModel

    property bool isMenuDisplayed: false
    signal action()

    function findGraphType()
    {
        if(main.isStepmill)
            return 2; // Speed only

        switch(workoutProgram.programIndex){
        case WorkoutProgram.NO_PROG:
        case WorkoutProgram.CUSTOM_ULTRA_PROG:
        case WorkoutProgram.CUSTOM_HRC_PROG:
        case WorkoutProgram.CUSTOM_C360_PROG:
        case WorkoutProgram.TREAD_CUSTOM_ULTRA_PROG:
        case WorkoutProgram.TREAD_CUSTOM_HRC_PROG:
        case WorkoutProgram.MOUNT_EVEREST_PROG:
        case WorkoutProgram.EIFFEL_TOWER_PROG:
        case WorkoutProgram.ALPINE_RANDOM_HILLS_PROG:
        case WorkoutProgram.GATEWAY_ARCH_PROG:
        case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
        case WorkoutProgram.MOUNT_RAINIER_PROG:
        case WorkoutProgram.SAVED_WORKOUT:
            return 1; // Both

        case WorkoutProgram.QUICK_START_PROG:
        case WorkoutProgram.TREAD_QUICK_START_PROG:
        case WorkoutProgram.MANUAL_PROG:
        case WorkoutProgram.DISTANCE_PROG:
        case WorkoutProgram.C360_QUICK_START_PROG:
        case WorkoutProgram.C360_PROG:
        case WorkoutProgram.C360_ARM_SCULPTOR_PROG:
        case WorkoutProgram.C360_LEG_SHAPER_PROG:
        case WorkoutProgram.C360_DEMO_PROG:
        case WorkoutProgram.C360_VIDEO_PROG:
        case WorkoutProgram.CALORIE_PROG:
        case WorkoutProgram.ROLLING_HILLS_PROG:
        case WorkoutProgram.SINGLE_HILL_PROG:
        case WorkoutProgram.HILL_INTERVALS_PROG:
        case WorkoutProgram.CARDIO_CHALLENGE_PROG:
        case WorkoutProgram.HRC_AEROBIC_PROG:
        case WorkoutProgram.HRC_WEIGHT_LOSS_PROG:
        case WorkoutProgram.HRC_TARGET_PROG:
        case WorkoutProgram.HRC_DISTANCE_PROG:
        case WorkoutProgram.HRC_INTERVALS_PROG:
        case WorkoutProgram.GLUTE_BUSTER_PROG:
        case WorkoutProgram.CUSTOM_HILLS_PROG:
        case WorkoutProgram.RANDOM_HILLS_PROG:
        case WorkoutProgram.TREAD_CALORIE_PROG:
        case WorkoutProgram.TREAD_CARDIO_CHALLENGE_PROG:
        case WorkoutProgram.TREAD_CUSTOM_HILLS_PROG:
        case WorkoutProgram.TREAD_DISTANCE_PROG:
        case WorkoutProgram.TREAD_GLUTE_BUSTER_PROG:
        case WorkoutProgram.TREAD_HILL_INTERVALS_PROG:
        case WorkoutProgram.TREAD_HRC_AEROBIC_PROG:
        case WorkoutProgram.TREAD_HRC_DISTANCE_PROG:
        case WorkoutProgram.TREAD_HRC_INTERVALS_PROG:
        case WorkoutProgram.TREAD_HRC_TARGET_PROG:
        case WorkoutProgram.TREAD_HRC_WEIGHT_LOSS_PROG:
        case WorkoutProgram.TREAD_MANUAL_PROG:
        case WorkoutProgram.TREAD_RANDOM_HILLS_PROG:
        case WorkoutProgram.TREAD_ROLLING_HILLS_PROG:
        case WorkoutProgram.TREAD_SINGLE_HILL_PROG:
        case WorkoutProgram.TREAD_WEIGHT_LOSS_HILLS_PROG:
        case WorkoutProgram.STRIDE_INTERVALS_PROG:
        case WorkoutProgram.STRIDE_DISTANCE_PROG:
        case WorkoutProgram.C360_LATERALTRAINER_PROG:
        case WorkoutProgram.C360_QUICK_START_LATERALTRAINER_PROG:
        case WorkoutProgram.QUAD_BURNER_PROG:
            return 0; // Incline/Workload Only

        case WorkoutProgram.PACE_RAMP_PROG:
        case WorkoutProgram.PACE_INTERVALS_PROG:
        case WorkoutProgram.LEG_SHAPER_PROG:
        case WorkoutProgram.WALK_AND_RUN_PROG:
        case WorkoutProgram.CUSTOM_PACE_PROG:
        case WorkoutProgram.FITNESS_TEST_PROG:
        case WorkoutProgram.TREAD_CUSTOM_SPEED_PROG:
        case WorkoutProgram.TREAD_LEG_SHAPER_PROG:
        case WorkoutProgram.TREAD_SPEED_INTERVALS_PROG:
        case WorkoutProgram.TREAD_SPEED_RAMP:
        case WorkoutProgram.TREAD_WALK_AND_RUN_PROG:
        case WorkoutProgram.TREAD_AIR_FORCE_MILITARY_FITNESS_TEST_PROG:
        case WorkoutProgram.TREAD_ARMY_MILITARY_FITNESS_TEST_PROG:
        case WorkoutProgram.TREAD_MARINES_MILITARY_FITNESS_TEST_PROG:
        case WorkoutProgram.TREAD_NAVY_MILITARY_FITNESS_TEST_PROG:
        case WorkoutProgram.TREAD_GERKIN_TEST_PROG:
        case WorkoutProgram.STEP_MANUAL_PROG:
            return 2; // Speed/Pace Only
        default:
            return 1
        }
    }

    Rectangle { id: backgroundReference
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        width: root.width
        height: root.height
        border.color: "grey"
        border.width: 2
        color: "transparent"
        visible: false
    }

    ListView {
        id: segmentView
        Component.onCompleted: contentX = 0
        width: root.width
        height: root.height
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        orientation: ListView.Horizontal
        model: root.model1
        delegate: viewDelegate
        interactive: false // root.editable
        clip: false
        currentIndex: activeSegment
        spacing:root.columnWidth * 0.5

//        flickableDirection: Flickable.HorizontalFlick
//        boundsBehavior: Flickable.StopAtBounds
//        maximumFlickVelocity: 2*height

        // Keep selected item in center of view
        highlightRangeMode: ListView.ApplyRange
        highlightFollowsCurrentItem: true
        preferredHighlightBegin: (width/2) - 60
        preferredHighlightEnd: (width/2) + 60
    }

    Component { id: viewDelegate

        Item { id: standardSegment

            function computeBarHeight1() {
                var bh = root.minBarHeight
                console.debug("segment mag: "+ segmentMagnitude)
                console.debug("magnitudeRangeMax: "+ root.model1.magnitudeRangeMax)
                if (hasNegativeValues) {
                    console.debug("segment mag: "+ segmentMagnitude)
                    bh = (segmentMagnitude / (root.model1.magnitudeRangeMax-root.model1.magnitudeRangeMin)) * root.maxBarHeight
                } else {
                    bh = (segmentMagnitude * maxBarHeight) / root.model1.magnitudeRangeMax
                }

                // very short bar for zero
                bh = Math.abs(bh) > root.minBarHeight ? bh : (bh >= 0 ? root.minBarHeight : -root.minBarHeight)
                console.debug("computeHeight1: "+bh)
                return bh
            }

            function computeBarHeight2() {
//                var bh = (root.model2.getProperty(index, "segmentLevel") * (root.topYPosition-root.axisPosition)) / root.model2.maxLevel
                //console.debug("DEBUG: index( "+ index)
                var bh = (root.model2.getProperty(index, "segmentMagnitude") * (root.topYPosition-root.axisPosition)) / root.model2.magnitudeRangeMax

                // very short bar for zero
                bh = bh > root.minBarHeight ? bh : root.minBarHeight
                console.debug("computeHeight2: "+bh)
                return bh
            }

            function getTextLabel2() {
                var segLevel = root.model2.getProperty(index, "segmentLevel")
                if (segLevel) {
                    if (root.model2.segmentType === 4) { // speed segment
                        var level = parseInt(segLevel)
                        var val = (machineinfo.speedTable[level - 1]/10.0)
                        val = options.metricUnits ? Globals.mileToKm(val) : val
                        return val.toFixed(1)
                    } else {
                        return segLevel
                    }
                } else {
                    return "?"
                }
            }

            width: 2*root.columnWidth
            height: root.height

            Rectangle {
                id: axis_line
                anchors.bottom: parent.bottom
                anchors.bottomMargin: root.axisPosition
                width: root.width
                height: 2
                color: "grey" //"transparent"
                visible: false
            }

            Rectangle {
                id: segmentBar1

                x: 0 // index * root.columnWidth
                y: root.height - root.axisPosition - (standardSegment.computeBarHeight1() > 0 ? standardSegment.computeBarHeight1() : 0)

                width: root.columnWidth
                height: Math.abs(standardSegment.computeBarHeight1())

                border.color: (root.active && root.editable && activeSegment == index && activeModel == 1) ? segmentColorInclineEdit : segmentColorIncline
                border.width: root.borderWidth
                color: (root.editable || activeSegment > index) ? border.color : "transparent"
                visible: root.graphDisplayMode <= 1 ? true : false

                MouseArea {
                    anchors.fill: parent
                    anchors.margins: -2
                    anchors.topMargin: parent.height - root.height
                    onClicked: {
                        activeSegment = index
                        root.activeModel = 1
                        buzzer.buttonPress()
                        root.action()
                    }
                    visible: enabled
                    enabled: root.editable && !isCustomSpeed //&& (activeModel == 2 || index !== activeSegment) // disable to not block slider for current item

                    Rectangle{
                        opacity: 0.3
                        color: "red"
                        anchors.fill: parent
                        visible: main.showStimulusArea
                    }
                }

                Item {
                    id: valueText1
                    visible: root.labelVisibility
                    anchors {
                        top: segmentBar1.bottom
                        topMargin: 5 * main.vScaleFactor
                        horizontalCenter: segmentBar1.horizontalCenter
                    }

                    NumberLabel {
                        id: valueTextValue1

                        font { pixelSize: 13 * main.vScaleFactor }
                        horizontalAlignment: Text.AlignHCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: {
                            if (segmentLevel) {

                                var level = parseInt(segmentLevel)
                                if (root.model1.segmentType === 3) { // incline segment
                                    return (machineinfo.inclineTable[level - 1]/10.0).toFixed(1)
                                }  else {
                                    return segmentLevel
                                }
                            } else {
                                return "?"
                            }
                        }
                        color: main.colorFontLight
                    }

                    Text {
                        id: valueTextLabel

                        anchors.top: valueTextValue1.bottom
                        font { pixelSize: 11 * main.vScaleFactor}
                        horizontalAlignment: Text.AlignHCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: {
                            if (segmentLevel) {
                                if (root.model1.segmentType === 3) { // incline segment
                                    return "%"
                                } else {
                                    return qsTr("LEVEL") + application.emptyString
                                }
                            } else return ""
                        }
                        color: main.colorFontLight
                    }
                }
            }

            Rectangle {
                id: segmentBar2

                x: 1.05*root.columnWidth // (index+1.05) * root.columnWidth
                anchors.bottom: axis_line.bottom
                width: root.columnWidth
                height: standardSegment.computeBarHeight2()
                border.color: (root.active && root.editable && activeSegment == index && activeModel == 2) ? segmentColorSpeedEdit : segmentColorSpeed
                border.width: root.borderWidth
                color: (root.editable || activeSegment > index) ? border.color : "transparent"
                visible: root.graphDisplayMode >= 1 ? true : false

                MouseArea {
                    anchors.fill: parent
                    anchors.margins: -2
                    anchors.topMargin: parent.height - root.height
                    onClicked: {
                        activeSegment = index
                        root.activeModel = 2
                        buzzer.buttonPress()
                        root.action()
                    }
                    visible: enabled
                    enabled: root.editable && !isCustomIncline//&& (index !== activeSegment || activeModel == 1) // disable to not block slider for current item

                    Rectangle{
                        opacity: 0.3
                        color: "red"
                        anchors.fill: parent
                        visible: main.showStimulusArea
                    }
                }

                Item {
                    id: valueText2
                    visible: root.labelVisibility
                    anchors {
                        top: segmentBar2.bottom
                        topMargin: 5 * main.vScaleFactor
                        left: parent.left
                        right: parent.right
                    }

                    NumberLabel {
                        id: valueTextValue2

                        font { pixelSize: 13 * main.vScaleFactor }
                        horizontalAlignment: Text.AlignHCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: standardSegment.getTextLabel2()
                        color: main.colorFontLight
                    }

                    Text {
                        id: valueTextLabel2

                        anchors.top: valueTextValue2.bottom
                        font { pixelSize: 11 * main.vScaleFactor }
                        horizontalAlignment: Text.AlignHCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        text: {
                            if (segmentLevel) {
                                if (root.model2.segmentType === 4) { // speed segment
                                    return main.isStepmill ? qsTr("SPM") : options.metricUnits ? qsTr("KPH") : qsTr("MPH")
                                } else {
                                    return qsTr("LEVEL") + application.emptyString
                                }
                            } else return ""
                        }
                        color: main.colorFontLight
                    }
                }

                Connections {
                    target: workoutManager

                    onSpeedChanged: {
                        segmentBar2.height= standardSegment.computeBarHeight2()

                    }
                    onPaceLevelChanged: {
                        segmentBar2.height= standardSegment.computeBarHeight2()
                    }
                }

                Connections {
                    target: root.model2
                    onDataChanged: {
                        segmentBar2.height= standardSegment.computeBarHeight2()
                        valueTextValue2.text= standardSegment.getTextLabel2()
                    }
                }
            }

            Rectangle { id: currentSegmentFill1
                anchors.bottom: segmentBar1.bottom
                anchors.bottomMargin: 0
                anchors.left: segmentBar1.left
                anchors.leftMargin: 0
                anchors.top: segmentBar1.top
                anchors.topMargin: 0
                width: root.columnWidth * root.segmentFill / 100

                color: segmentBar1.border.color
                visible: (activeSegment == index ? true : false) && (root.graphDisplayMode <= 1 ? true : false)
            }

            Rectangle { id: currentSegmentFill2
                anchors.bottom: segmentBar2.bottom
                anchors.bottomMargin: 0
                anchors.left: segmentBar2.left
                anchors.leftMargin: 0
                anchors.top: segmentBar2.top ; anchors.topMargin: 0
                width: root.columnWidth * root.segmentFill / 100

                color: segmentBar2.border.color
                visible: (activeSegment == index ? true : false) && (root.graphDisplayMode >= 1 ? true : false)
            }
        }
    }

    Row {
        y: root.editable || root.isSummary ? root.yPosition - legendSize : root.height - 0.75*root.yPosition
        x: 0
        spacing: 0.25*legendSize
        height: legendSize
        LayoutMirroring.enabled: main.rightToLeft
        LayoutMirroring.childrenInherit: true

        Rectangle { id: legend1
            width: legendSize
            height: legendSize
            color: root.graphDisplayMode == 1 ? segmentColorIncline : "transparent"
            visible: root.graphDisplayMode <= 1 ? true : false
        }

        Text { id: legendLabel1
            visible: root.graphDisplayMode <= 1 ? true : false
            width: root.graphDisplayMode == 1 ? 0.1 * root.width : 0
            anchors.verticalCenter: parent.verticalCenter
            text: ((main.isTreadmill || main.isStepmill) ? qsTr("INCLINE") : qsTr("WORKLOAD")) + application.emptyString
            color: main.colorFontLight
            horizontalAlignment: Text.AlignLeft
            font.pointSize: legendFontSize
            font.bold: false
        }

        Rectangle { id: legend2
            width: legendSize
            height: legendSize
            color: root.graphDisplayMode == 1 ? segmentColorSpeed : "transparent"
            visible: root.graphDisplayMode >= 1 ? true : false
        }

        Text { id: legendLabel2
            visible: root.graphDisplayMode >= 1 ? true : false
            width: root.graphDisplayMode == 1 ? 0.1 * root.width : 0
            anchors.verticalCenter: parent.verticalCenter
            text: ((main.isTreadmill || main.isStepmill) ? qsTr("SPEED") : qsTr("PACE")) + application.emptyString
            color: main.colorFontLight
            horizontalAlignment: Text.AlignLeft
            font.pointSize: legendFontSize
            font.bold: false
        }
    }
}
