import WorkoutRunData 1.0
import WorkoutManager 1.0
import QtQuick 2.0
import "../string_format.js" as Format
import Components 1.0
import "../"

Item {
    id: root

    property int screenHeight: parent.height
    property int screenWidth: parent.width

    property bool isQuickSpeedKeysDisplayed: false
    property bool isQuickStrideKeysDisplayed: false
    property int currentStride

    property string machineType: machineinfo.productTypeString
    property bool bikeModeEnabled: false

    signal inclinePressed()
    signal speedPressed()
    signal stopPressed()
    signal strideSelected(int stride)

    height: screenHeight * 0.20
    width: screenWidth

    Component.onCompleted: reset();

    /**
     * Resets the toolbar to its initial state
     */
    function reset() {
        isQuickSpeedKeysDisplayed = false
        isQuickStrideKeysDisplayed = false
//        isHomeSelected = false
    }

    property string stringTargetSpeed:  {
        var tempSpeed;

        if(main.isStepmill) {
            tempSpeed = workoutManager.treadSpeedGUI
            return tempSpeed.toFixed(0)
        } else if (workoutScreen.unitState === "ENGLISH") {
            tempSpeed = workoutManager.treadSpeedGUI / 10.0
            return tempSpeed.toFixed(1)
        } else {
            tempSpeed = workoutManager.treadSpeedMetricGUI / 10.0
            return tempSpeed.toFixed(1)
        }
    }

    property string stringTargetIncline: (workoutManager.treadInclineGUI / 10.0).toFixed(1)

    function setTreadmillSpeed(speed) {
        if(workoutScreen.unitState === "ENGLISH") {
            workoutManager.treadSpeed = speed * 10
        } else {
            workoutManager.setTreadSpeedMetric(parseFloat(speed) * 10)
        }
    }

    function setStepmillSpeed(speed) {
        workoutManager.treadSpeed = speed
    }

    property string stringLevel: workoutManager.workloadLevel
    property string stringRPM: aWorkoutRunData.rpm

    onIsQuickStrideKeysDisplayedChanged: {
        if (isQuickStrideKeysDisplayed) {
            // Show quick stride keys for 15 seconds
            hideQuickKeysTimer.restart()
        }
    }

    Timer {
        id: hideQuickKeysTimer
        interval: 15000
        running: false
        repeat: false
        onTriggered: {
            if (isQuickStrideKeysDisplayed) {
                isQuickStrideKeysDisplayed = false
            }
        }
    }

    Rectangle {
        color: main.colorToolbarBackground
        anchors.fill: parent
    }

    Component {
        id: control

        Item {
            id: controlRoot
            property string stringLabel: "label"
            property string stringValue: "value"
            property string stringUnit: "unit"
            property string stringMiddleLabel: ""
            property bool controlEnabled: true
            property bool useStimulusArea: true

            signal toggleCalculator()
            signal controlUpClicked()
            signal controlDownClicked()
            signal controlUpReleased()
            signal controlUpFirstPress()

            Row {
                height: parent.height
                width: parent.width

                MouseArea {
                    onClicked: {
                        toggleCalculator()
                        buzzer.buttonPress()
                    }

                    width:  childrenRect.width
                    height: childrenRect.height

                    Rectangle {
                        height: parent.parent.height
                        width: parent.parent.width / 2
                        color: (main.showStimulusArea && useStimulusArea) ? main.colorGenericStimulusArea : main.colorButtonBackground

                        Column {
                            id: info
                            anchors.fill: parent

                            Item {
                                id: labelRectangle
                                height: (parent.height - valueRectangle.height) * 0.5
                                width: parent.width

                                TextLabel {
                                    id: label
                                    text: stringLabel
                                    anchors.fill: parent
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignBottom
                                    font.pixelSize: parent.height * 0.3
                                    isHairline: true
                                }
                            }

                            Item {
                                id: valueRectangle
                                height: parent.height * 0.4
                                width: parent.width

                                NumberLabel {
                                    id: value
                                    text: stringValue
                                    color: main.colorFontLight
                                    anchors.fill: parent
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    font.pixelSize: parent.height
                                }
                            }

                            Item {
                                id: unitRectangle
                                height: (parent.height - valueRectangle.height) * 0.5
                                width: parent.width

                                TextLabel {
                                    id: unit
                                    text: stringUnit
                                    anchors.fill: parent
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignTop
                                    font.pixelSize: parent.height * 0.3
                                    isHairline: true
                                }
                            }
                        }
                    }
                }

                Column {
                    height: parent.height
                    width: parent.width / 2
                    visible: controlRoot.controlEnabled

                    Item {
                        id: upControlItem
                        opacity: 1
                        height: (parent.height - middleLabel.height) / 2
                        width: parent.width

                        ImageButton {
                            id: upControl
                            width: 91 * main.hScaleFactor
                            height: 52 * main.vScaleFactor
                            anchors.centerIn: parent
                            useParentContainerSize: true
                            downimage: ""
                            upimage: ""
                            icon: "../graphics/ui3_0/LowerControlBar/Control_Up.png"
//                            sourceName: "LowerControlBar/Control_Up";
                            repeat: true
                            onClicked: controlUpClicked()
                            onIsDownChanged: {
                                if(isDown === false) {
                                    controlUpReleased()
                                } else {
                                    controlUpFirstPress()
                                }
                            }

                            mouseAreaScale: 1.25
                        }
                    }

                    TextLabel {
                        id: middleLabel
                        height: parent.height * 0.1
                        width: parent.width
                        text: stringMiddleLabel
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        font.pixelSize: height
                    }


                    Item {
                        id: downControlItem
                        opacity: 1
                        height: (parent.height - middleLabel.height) / 2
                        width: parent.width

                        ImageButton {
                            id: downControl
                            width: 91 * main.hScaleFactor
                            height: 52 * main.vScaleFactor
                            anchors.centerIn: parent
                            useParentContainerSize: true
                            downimage: ""
                            upimage: ""
                            icon: "../graphics/ui3_0/LowerControlBar/Control_Down.png"
//                            icon.sourceName: "LowerControlBar/Control_Down";
                            repeat: true
                            onClicked: controlDownClicked()
                            mouseAreaScale: 1.25
                        }
                    }
                }
            }
        }
    }

    Row {
        anchors.fill: parent

        property real stopButtonWidthPercent: 0.22
        property real controlWidthPercent: 0.18
        property real quickKeyWidthPercent: 0.08
        property real spacerWidthPercent: 0.06
        property real spacerWidthPercent2: 0.10
        property real spacerWidthPercent3: 0.05
        property real speedGuagePercent: 0.075
        opacity: enabled ? 1.0 : 0.5

        Item {
            id: stopButtonItem
            height: parent.height
            width: parent.width * parent.stopButtonWidthPercent

            Rectangle {
                id: stopButton
                border.color: main.colorUIBorder
                border.width: main.widthBorder
                color: main.colorButtonBackground
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                }
                width: 139 * main.hScaleFactor
                height: 113 * main.vScaleFactor
                radius: main.defaultRadius

                ScalableImage {
                    id: stopButtonImage
                    anchors.fill: parent
                    anchors.margins: 22 * main.scaleFactor
                    fillMode: Image.PreserveAspectFit
                    sourceName: "LowerControlBar/Stop_Pause_Icon";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        root.stopPressed()
                        workoutScreen.stopButtonPressedHandler()
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }

        Item {
            id: spacerItem
            height: parent.height
            width: parent.width * parent.spacerWidthPercent
        }

        Row {
            id: speedPresetsRow
            height: parent.height
            width: inclineControl.width + spacerItem2.width
            visible: isQuickSpeedKeysDisplayed === true

            Item {
                id: spacerItemSpeed
                height: parent.height
                width: spacerItem2.width
            }

            Repeater {
                model: main.isStepmill ? stepSpeedsModel : (workoutScreen.unitState === "ENGLISH" ? speedsModel : km_speedsModel)
                delegate: speedDelegate
            }

            // To Do: Convert to steps per minute quick keys for stepmill

            ListModel {
                id: stepSpeedsModel
                ListElement { speed: 35 ; label: "35"; units: qsTr("SPM") }
                ListElement { speed: 57 ; label: "57"; units: qsTr("SPM") }
                ListElement { speed: 80 ; label: "80"; units: qsTr("SPM") }
            }

            ListModel {
                id: speedsModel
                ListElement { speed: 3.0 ; label: "3"; units: qsTr("MPH") }
                ListElement { speed: 5.0 ; label: "5"; units: qsTr("MPH") }
                ListElement { speed: 7.0 ; label: "7"; units: qsTr("MPH") }
            }

            ListModel {
                id: km_speedsModel
                ListElement { speed: 5.0  ; label: "5"  ; units: qsTr("KPH") }
                ListElement { speed: 8.0  ; label: "8"  ; units: qsTr("KPH") }
                ListElement { speed: 11.0 ; label: "11" ; units: qsTr("KPH") }
            }

            Component {
                id: speedDelegate
                Item {
                    height: speedPresetsRow.height
                    width: speedPresetsRow.width / 4

                    Rectangle {
                        border.color: main.colorUIBorder
                        border.width: main.widthBorder
                        color: main.colorButtonBackground
                        width: 90 * main.hScaleFactor
                        height: 113 * main.vScaleFactor
                        radius: main.defaultRadius
                        anchors.centerIn: parent

                        NumberLabel {
                            text: model.label
                            anchors.centerIn: parent
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            font.pixelSize: parent.height * 0.4
                        }

                        Text {
                            color: main.colorFontLight
                            text: model.units
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 7 * main.vScaleFactor
                            anchors.horizontalCenter: parent.horizontalCenter
                            font.bold: false
                            font.pixelSize: parent.height * 0.2
                            font.family: "Montserrat Hairline"
                        }

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                if(main.isStepmill)
                                    setStepmillSpeed(model.speed)
                                else
                                    setTreadmillSpeed(model.speed)

                                buzzer.buttonPress()
                            }

                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }
                }
            }
        }

        Row {
            id: stridePresetsRow
            height: parent.height
            width: levelControl.width + spacerItem2.width
            visible: isQuickStrideKeysDisplayed === true

            Item {
                id: spacerItemStride
                height: parent.height
                width: spacerItem2.width
            }

            Repeater {
                model: strideModel
                delegate: strideDelegate
            }

            ListModel {
                id: strideModel
                ListElement { stride: 13 ; label: "STEP"; icon: "AdjStride/AS_Step_Icon"; }
                ListElement { stride: 21 ; label: "ELLIPSE"; icon: "AdjStride/AS_Ellipse_Icon"; }
                ListElement { stride: 30 ; label: "RUN"; icon: "AdjStride/AS_Run_Icon"; }
            }

            Component {
                id: strideDelegate
                Item {
                    height: stridePresetsRow.height
                    width: stridePresetsRow.width / 4

                    Rectangle {
                        border.color: main.colorUIBorder
                        border.width: main.widthBorder
                        color: root.currentStride === model.stride ? main.colorHighlightBackground : main.colorButtonBackground
                        width: 90 * main.hScaleFactor
                        height: 113 * main.vScaleFactor
                        radius: main.defaultRadius
                        anchors.centerIn: parent

                        ScalableImage {
                            anchors.centerIn: parent
                            anchors.verticalCenterOffset: -10 * main.vScaleFactor
                            width: 0.9 * parent.width
                            height: 52 * main.vScaleFactor
                            fillMode: Image.PreserveAspectFit

                            sourceName: model.icon
                        }

                        Text {
                            color: main.colorFontLight
                            text: model.label
                            anchors.horizontalCenter: parent.horizontalCenter
                            anchors.bottom: parent.bottom
                            anchors.bottomMargin: 12*main.scaleFactor
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignBottom
                            font.bold: false
                            font.pixelSize: parent.height * 0.12
                            font.family: "Montserrat Hairline"
                        }

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                hideQuickKeysTimer.restart()
                                root.strideSelected(model.stride)
                                buzzer.buttonPress()
                            }

                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }
                }
            }
        }

        Loader {
            id: inclineControl
            sourceComponent: control
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height
            width: parent.width * parent.controlWidthPercent
            visible: isQuickSpeedKeysDisplayed === false && main.isTreadmill

            Binding {
                target: inclineControl.item
                property: "stringLabel"
                value: qsTr("INCLINE") + application.emptyString
            }

            Binding {
                target: inclineControl.item
                property: "stringValue"
                value: stringTargetIncline
            }

            Binding {
                target: inclineControl.item
                property: "stringUnit"
                value: "%"
            }

            Binding {
                target: inclineControl.item
                property: "useStimulusArea"
                value: true
            }

            Connections {
                target: inclineControl.item
                onToggleCalculator: inclinePressed()
                onControlUpClicked: workoutManager.increaseIncline()
                onControlDownClicked: workoutManager.decreaseIncline()
            }
        }

        Loader {
            id: levelControl
            sourceComponent: control
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height
            width: parent.width * parent.controlWidthPercent
            visible: isQuickSpeedKeysDisplayed === false && !main.isTreadmill && !main.isStepmill && isQuickStrideKeysDisplayed === false

            Binding {
                target: levelControl.item
                property: "stringLabel"
                value: {
                    if(root.bikeModeEnabled)
                        return qsTr("GEAR")
                    else
                        return qsTr("WORKLOAD")
                }
            }

            Binding {
                target: levelControl.item
                property: "stringValue"
                value: stringLevel
            }

            Binding {
                target: levelControl.item
                property: "stringUnit"
                value: qsTr(root.bikeModeEnabled ? "" : "LEVEL") + application.emptyString
            }

            Binding {
                target: levelControl.item
                property: "useStimulusArea"
                value: false
            }

            Connections {
                target: levelControl.item
                onControlUpClicked: keymapper.keyFunctionHandler(7)
                onControlDownClicked: keymapper.keyFunctionHandler(8)
            }
        }

        Item {
            id: spacerItem2
            height: parent.height
            width: parent.width * parent.spacerWidthPercent2
            visible: isQuickSpeedKeysDisplayed === false && isQuickStrideKeysDisplayed === false
        }

        Item {
            id: quickSpeedToggleButtonItem
            height: parent.height
            width: parent.width * parent.quickKeyWidthPercent
            visible: main.isTreadmill || main.isStepmill

            Rectangle {
                id: quickSpeedToggleButton
                border.color: isQuickSpeedKeysDisplayed ? main.colorUIBorder : "transparent"
                border.width: main.widthBorder
                radius: main.defaultRadius
                color: main.colorButtonBackground
                anchors.left: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                width: 76 * main.hScaleFactor
                height: 64 * main.vScaleFactor

                ScalableImage {
                    id: quickSpeedToggleButtonIcon
                    anchors.fill: parent
                    anchors.centerIn: parent
                    anchors.margins: 10 * main.scaleFactor
                    fillMode: Image.PreserveAspectFit

                    sourceName: "WF/WF_Button_Speed_Workout";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        isQuickSpeedKeysDisplayed = !isQuickSpeedKeysDisplayed
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }

        Item {
            id: quickStrideToggleButtonItem
            height: parent.height
            width: parent.width * parent.quickKeyWidthPercent
            visible: machineType === "Adjustable Stride Elliptical"

            Rectangle {
                id: quickStrideToggleButton
                border.color: isQuickStrideKeysDisplayed ? main.colorUIBorder : "transparent"
                border.width: main.widthBorder
                radius: main.defaultRadius
                color: main.colorButtonBackground
                anchors.left: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                width: 76 * main.hScaleFactor
                height: 64 * main.vScaleFactor

                ScalableImage {
                    id: quickStrideToggleButtonIcon
                    anchors.fill: parent
                    anchors.centerIn: parent
                    anchors.margins: 10 * main.scaleFactor
                    fillMode: Image.PreserveAspectFit

                    sourceName: "AdjStride/AS_Run_Icon";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        isQuickStrideKeysDisplayed = !isQuickStrideKeysDisplayed
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }

        Item {
            id: intensifierToggleButtonItem
            height: parent.height
            width: parent.width * parent.quickKeyWidthPercent
            visible: main.isLateralTrainer && intensifierToggleButton.allowed

            Rectangle {
                id: intensifierToggleButton
                border.color: intensifierActive ? main.colorUIBorder : "transparent"
                border.width: main.widthBorder
                radius: main.defaultRadius
                color: main.colorButtonBackground
                anchors.left: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                width: 60 * main.hScaleFactor
                height: 60 * main.vScaleFactor

                property bool allowed: !workoutProgram.isHRCProgram &&
                         !workoutProgram.isFitnessTest &&
                         !workoutManager.cruiseControlEnabled
                property bool intensifierActive: workoutManager.intensifierEnabled

                ScalableImage {
                    id: intensifierToggleButtonIcon
                    anchors.fill: parent
                    anchors.centerIn: parent
                    anchors.margins: 10 * main.scaleFactor
                    fillMode: Image.PreserveAspectFit

                    sourceName: "LowerControlBar/Intensifier_Icon";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        workoutManager.intensifierEnabled = !workoutManager.intensifierEnabled
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }

        Item {
            id: spacerItemSpeedGuage
            height: parent.height
            width: parent.width * parent.speedGuagePercent
            visible: !root.bikeModeEnabled
        }

        ScalableImage {
            id: bikeModeMPHGauge
            visible: root.bikeModeEnabled
            height: parent.height
            width: parent.width * parent.speedGuagePercent
            sourceName: "BikeMode/SPEED_METER_bikemode";
            anchors.margins: 15 * main.scaleFactor
            anchors.verticalCenter: parent.verticalCenter
            fillMode: Image.PreserveAspectFit

            NumberLabel {
                id:value
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                text: {
                    if (workoutScreen.unitState === "ENGLISH") {
                        aWorkoutRunData.bikeSpeedMph.toFixed(1)
                    } else {
                        Globals.mileToKm(aWorkoutRunData.bikeSpeedMph).toFixed(1)
                    }
                }
                font.pixelSize: 26 * main.vScaleFactor
            }

            TextLabel {
                id: label
                anchors.top: value.bottom
                anchors.topMargin: -2 * main.vScaleFactor
                anchors.horizontalCenter: parent.horizontalCenter
                text: workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH")
                font.pixelSize: 10 * main.vScaleFactor
            }
        }

        Loader {
            id: speedControl
            sourceComponent: control
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height
            width: parent.width * parent.controlWidthPercent
            visible: main.isTreadmill || main.isStepmill

            Binding {
                target: speedControl.item
                property: "stringLabel"
                value: qsTr("SPEED") + application.emptyString
            }

            Binding {
                target: speedControl.item
                property: "stringValue"
                value: stringTargetSpeed
            }

            // To Do: update for steps per minute for stepmill
            Binding {
                target: speedControl.item
                property: "stringUnit"
                value: main.isStepmill ? qsTr("SPM") : (workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH"))
            }

            Binding {
                target: speedControl.item
                property: "useStimulusArea"
                value: true
            }

            Connections {
                target: speedControl.item
                onToggleCalculator: speedPressed()
                onControlUpClicked: {
                    if(workoutScreen.unitState === "ENGLISH") {
                        workoutManager.increaseSpeed(workoutManager.withLimit);
                    } else {
                        workoutManager.increaseSpeedMetric(workoutManager.withLimit);
                    }
                }
                onControlDownClicked: {
                    if(workoutScreen.unitState === "ENGLISH") {
                        workoutManager.decreaseSpeed(false);
                    } else {
                        workoutManager.decreaseSpeedMetric(false);
                    }
                }
                onControlUpFirstPress: {
                    workoutManager.speedUpEnabled = true
                }
                onControlUpReleased: {
                    workoutManager.speedUpEnabled = false
                }
            }
        }

        Loader {
            id: paceControl
            sourceComponent: control
            anchors.verticalCenter: parent.verticalCenter
            height: parent.height
            width: parent.width * parent.controlWidthPercent + (paceControl.isPaceSetter ? 0.05 * main.screenWidth : 0)
            visible: !main.isTreadmill && !main.isStepmill

            property bool isPaceSetter: workoutProgram.displayDataVars.paceSetterAvailable

            Binding {
                target: paceControl.item
                property: "stringLabel"
                value: root.bikeModeEnabled ? qsTr("CADENCE") : qsTr("PACE") + application.emptyString
            }

            Binding {
                target: paceControl.item
                property: "stringValue"
                value: stringRPM
            }

            Binding {
                target: paceControl.item
                property: "stringUnit"
                value: qsTr("RPM")
            }

            Binding {
                target: paceControl.item
                property: "stringMiddleLabel"
                value: paceControl.isPaceSetter ? "PACESETTER" : ""
            }

            Binding {
                target: paceControl.item
                property: "controlEnabled"
                value: paceControl.isPaceSetter
            }

            Binding {
                target: paceControl.item
                property: "useStimulusArea"
                value: false
            }

            Connections {
                target: paceControl.item
                onControlUpClicked: workoutManager.increasePaceLevel()
                onControlDownClicked: workoutManager.decreasePaceLevel()
            }
        }

        Item {
            id: paceLevelMeter

            property int rpmMax: 120
            property int rpmMin: 40

            property int paceTarget: workoutManager.paceTarget < rpmMin ? rpmMin : (workoutManager.paceTarget > rpmMax ? rpmMax : workoutManager.paceTarget)
            property int adjustedRPM: aWorkoutRunData.rpm < rpmMin ? rpmMin : (aWorkoutRunData.rpm  > rpmMax ? rpmMax : aWorkoutRunData.rpm )

            property real paceTargetPercent: 0.95*(paceTarget - rpmMin)/(rpmMax - rpmMin) + 0.05
            property real rpmPercent: 0.95*(adjustedRPM - rpmMin)/(rpmMax - rpmMin) + 0.05

            property real paceTargetHeight: paceTargetPercent * height
            property real rpmHeight: rpmPercent * height

            anchors.centerIn: paceControl
            height: 0.8*parent.height
            visible: paceControl.isPaceSetter

            Rectangle {
                id: currentRPMFill

                width: levelHash.width
                height: paceLevelMeter.rpmHeight
                anchors.left: parent.horizontalCenter
                anchors.bottom: parent.bottom
                color: main.colorUIBorder
            }

            Column {
                id: levelHash

                property int lineHeight: Math.ceil(1*main.vScaleFactor)

                spacing: 3*lineHeight
                width: 15*main.hScaleFactor
                height: paceLevelMeter.height
                anchors.left: parent.horizontalCenter
                anchors.bottom: parent.bottom

                Repeater {
                    model: 20

                    Rectangle {
                        width: levelHash.width
                        height: levelHash.lineHeight
                        color: "#4D4B45"
                    }
                }
            }

            ScalableImage {
                id: goalLevelArrow

                anchors.right: parent.horizontalCenter
                anchors.rightMargin: 1
                anchors.bottom: parent.bottom
                anchors.bottomMargin: paceLevelMeter.paceTargetHeight - height/2

                width: 30 * main.hScaleFactor
                height: 20 * main.vScaleFactor

                sourceName: "LowerControlBar/PaceLevel_Arrow";

                NumberLabel {
                    id: goalLevelText
                    property string stringPaceLevel: "00"

                    text: stringPaceLevel
                    anchors.fill: parent
                    anchors.rightMargin: 10 * main.hScaleFactor

                    color: main.colorFontDark
                    font.pixelSize: height * 0.75
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                }

                Binding {
                    target: goalLevelText
                    property: "stringPaceLevel"
//                    value: workoutManager.paceLevel
                    value: workoutManager.paceTarget
                }
            }
        }

        Item {
            id: spacerItem3
            height: parent.height
            width: parent.width * parent.spacerWidthPercent3
        }
    }
}

