import WorkoutRunData 1.0
import WorkoutManager 1.0
import QtQuick 2.0
import "../string_format.js" as Format
import Components 1.0

Item {
    id: root

    property int screenHeight: parent.height
    property int screenWidth: parent.width

    property bool isMinimizedButtonPressed: false
    property bool isToolsButtonPressed: false

    height: screenHeight * 84/1080
    width: screenWidth

    signal minimizeButtonPressed()
    signal toolsButtonPressed()

    onMinimizeButtonPressed: isMinimizedButtonPressed = !isMinimizedButtonPressed
    
    Component.onCompleted: reset();
       
    /**
     * Resets the toolbar to its initial state
     */
    function reset() {
        isMinimizedButtonPressed = false;
    }

    Rectangle {
        color: isMinimizedButtonPressed === false ? main.colorToolbarBackground : "transparent"
        anchors.fill: parent
    }

    Rectangle {
        id: minimizeButton
        visible: false // Hide minimize functionality for now (per David T. 11/28/17)
        color: "transparent"
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        anchors.leftMargin: 25 * main.hScaleFactor
        width: 49 * main.hScaleFactor
        height: 45 * main.vScaleFactor

        ScalableImage {
            id: minimizeIcon
            anchors.fill: parent
            anchors.margins: 13 * main.vScaleFactor
            anchors.centerIn: parent
            fillMode: Image.PreserveAspectFit
            sourceName: isMinimizedButtonPressed ? "UpperToolBar/maximize_icon" : "UpperToolBar/minimize_icon";
        }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                minimizeButtonPressed()
                buzzer.buttonPress()
            }

            Rectangle {
                anchors.fill: parent
                color: main.colorGenericStimulusArea
                opacity: 0.3
                visible: main.showStimulusArea
            }
        }
    }

    Item {
        id: toolsButton
        visible: isMinimizedButtonPressed === false
        anchors.verticalCenter: parent.verticalCenter
        anchors.right: parent.right
        anchors.rightMargin: 20 * main.hScaleFactor
        width: 53 * main.hScaleFactor
        height: 45 * main.vScaleFactor

        Rectangle {
            id: highlight
            visible: isToolsButtonPressed
            anchors.fill: parent
            color: "transparent"
            border.color: main.colorUIBorder
            border.width: main.widthBorder
            radius: main.defaultRadius
        }

        ScalableImage {
            id: toolsIcon
            anchors.fill: parent
            anchors.centerIn: parent
            anchors.margins: 13 * main.vScaleFactor
            fillMode: Image.PreserveAspectFit
            sourceName: "ToolsMenu/menu_icon";
        }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                toolsButtonPressed()
                buzzer.buttonPress()
            }

            Rectangle {
                anchors.fill: parent
                color: main.colorGenericStimulusArea
                opacity: 0.3
                visible: main.showStimulusArea
            }
        }
    }

    Item {
        id: enterWeightToolTip
        visible: isMinimizedButtonPressed === false
        height: parent.height
        width: parent.width * 0.14
        anchors.verticalCenter: parent.verticalCenter
        anchors.right: toolsButton.left
    }
}
