// import QtQuick 2.4 // to target S60 5th Edition or Maemo 5
import QtQuick 2.4
import "../views.js" as TheViews
import "../"

Item {
    id: root

    property bool isMenuDisplayed: false
    property alias isMenuMoving: sceneView.isMenuMoving
    signal action()

    Component.onDestruction: {
        audioManager.virtActivePlaying = false
    }

    // change mute status when there's a change in audio priorities
    Connections {
        target: audioManager

        onVirtActiveAllowedChanged: {
            sceneView.muted = !audioManager.virtActiveAllowed
        }

        onVirtActiveVideoAllowedChanged: {
            if( !audioManager.virtActiveVideoAllowed )
                sceneView.stopVideo()
        }

    }

    function shown() {
        audioManager.btRequest()
        sceneView.startVideo()
        sceneView.muted = !audioManager.virtActiveAllowed

        workoutViewContainer.persistMenu = true;
    }

    function hidden() {
        workoutViewContainer.persistMenu = false;
        sceneView.pause()
    }

    SceneView {
        id: sceneView

        anchors.fill: parent
        muted: audioManager.virtActiveAllowed
        isMenuDisplayed: root.isMenuDisplayed

        visible: true


        onPlayingChanged: {
            // request to play audio when playback starts.
            audioManager.virtActivePlaying = playing
        }

        Connections{
            target: workoutScreen
            onWorkoutStopped: sceneView.stopVideo()
        }

        onAction: root.action()
    }
}
