import QtQuick 2.4
import QtQuick.Layouts 1.3
import VLCVideo 1.0
import ".."
import Components 1.0

Item {
    id:root

    property bool viewShowing: false
    property string channelID: ""
    property alias channelIndex: channelMenu.currentIndex
    property alias channelCount: channelMenu.count
    property bool isSelected: currentViewName === "TV_VIEW"
    property bool tunedToCabChannel: tv.tunedToCab()

    property bool isMenuDisplayed: false
    property alias isMenuMoving: channelMenu.moving

    objectName: "TV_VIEW_CONTAINER"

    signal stopped()
    signal menuButtonPressed()
    signal action()

    function viewdestruct() {
        tv.stop();
    }

    Connections {
        target: tv
        onTuningChanged: {
            if(tv.tuning === false) {
                if(root.viewShowing === false) {
                    vlcMute.setMute(1)
                }
            }
        }
    }

    Timer {
        interval: tvTestHud.channelChangeTime
        repeat: true
        running: !channelChangeTimeSlider.off
        onTriggered: {
            if(!tv.usbReseting) {
                root.channelIndex = (root.channelIndex + 1) % channelCount
                tv.channelUp()
                tvTestHud.count++
            }
        }
    }

    Timer {
      interval: tvTestHud.usbFaultTime
      repeat: true
      running: !channelChangeTimeSlider.off && tvTestHud.usbFaultTime > 0
      onTriggered: {
          if(!tv.usbReseting) {
              tv.usbError()
              tvTestHud.usbcount++
          }
      }
    }

    function reset() {
        channelMenu.currentIndex = -1;
    }

    function shown() {
        tv.on = true
        root.viewShowing = true;
        workoutViewContainer.persistMenu = true;

        // request to play audio when switching to this view
        if (tv.isIpodInput) {
            tv.restoreChannel();
        } else {
            if(channelMenu.currentIndex < 0) {
                tv.channelUp()
                channelMenu.currentIndex = 0
            }
        }

        tv.muted = !audioManager.tvAllowed;
        audioManager.tvRequest();
    }

    function hidden() {
        tv.on = false
        root.viewShowing = false;
        workoutViewContainer.persistMenu = false;
    }

    function stop() {
        root.stopped();
        tv.stop();
        root.reset();
    }

    Component.onCompleted: {
        if (tv.currentChannel >= 0) {
            var chanid = tv.channelList.channels.get(tv.currentChannel).id
            channelSelected(chanid)
        }
    }

    Connections{
        target: workoutScreen
        onWorkoutStopped: root.stop()
    }

    Rectangle {
        color: main.colorDialogBackground
        anchors.fill: parent
        border.color: main.colorUIBorder
        border.width: main.widthBorder
    }

    RowLayout {
        anchors.fill: parent
        anchors.margins: 1.5 * main.widthBorder
        spacing: 0

        Item {
            Layout.preferredWidth: root.width*0.15
            Layout.fillHeight: true
            Layout.margins: 10 * main.scaleFactor
            Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter
            visible: !root.fullscreen && isMenuDisplayed

            ColumnLayout {
                anchors.fill: parent
                spacing: 5 * main.vScaleFactor

                Item { // channel list area
                    id: listArea
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    Layout.alignment: Qt.AlignHCenter | Qt.AlignTop

                    ListView {
                        id: channelMenu
                        anchors.fill: parent
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        currentIndex: -1
                        model: tv.channelList.channels
                        delegate: menuButtonDelegate
                        spacing: 8 * main.vScaleFactor
                        clip: true
                    }
                }

                Item { // media buttons
                    id: controlArea
                    Layout.fillWidth: true
                    Layout.preferredHeight: 0.8*parent.width / 2 * controlGrid.rows
                    Layout.alignment: Qt.AlignHCenter | Qt.AlignBottom

                    GridLayout {
                        id: controlGrid
                        anchors.fill: parent
                        rows: options.cabEnabled ? 3 : 2
                        rowSpacing: 7 * main.hScaleFactor
                        columns: 2
                        columnSpacing: 7 * main.vScaleFactor

                        property real delegateWidth: (width-columnSpacing)/columns
                        property real delegateHeight: (height-rowSpacing)/rows

                        Repeater {
                            model: options.cabEnabled ? controlsListCab : controlsList
                            delegate: controlButtonDelegate
                        }

                        Component {
                            id: controlButtonDelegate

                            Item {
                                Layout.preferredWidth: controlGrid.delegateWidth
                                Layout.preferredHeight: controlGrid.delegateHeight

                                StandardButton {
                                    anchors.fill: parent
                                    sourceName: model.iconPath
                                    buttonText: qsTr(model.label) + application.emptyString
//                                    buttonText: model.name ===  "CloseCaption" ? qsTr("Track #") + tv.captionTrack  : qsTr(model.label)
//                                    label.font.pixelSize: 10 * main.vScaleFactor
//                                    wideImage: true
                                    selected: (volume_controls.muted && model.name === "VolumeMute") || (tv.captionTrack>0 && model.name ===  "CloseCaption")
                                    color: selected ? main.colorSelected : main.colorDialogBackground
                                    onClicked: {
                                        controlClicked(model.name)
                                        root.action()
                                    }
                                }
                            }
                        }
                    } // end GridLayout
                } // end Item
            } // end ColumnLayout
        } // end Item

        Item { // TV area
            id: videoViewBackground
            Layout.fillWidth: true
            Layout.fillHeight: true

            VLCSurface {
                id: tvView
                player: tv.player
                playing: !player.fullscreen && root.viewShowing && (tv.playing || tv.tuning)
                aspect: tv.aspect
                anchors.fill: parent
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                onPlayingChanged: audioManager.rebuildSourceList()

                Rectangle {
                    color: "black"
                    visible: tv.tuningTimedOut && !tv.playing
                    anchors.fill: parent
                }

                Text {
                    anchors.centerIn: parent
                    color: "white"
                    visible: (tv.usbResetInProgress|| tv.tuningTimedOut) && !tv.playing
                    text: (tv.usbResetInProgress ? qsTr("TV may be unavailable for the remainder of the workout.") : qsTr("The current channel may be temporarily unavailable.")) + application.emptyString
                }

            	DiagnosticOverlay {
                    id: diagOverlay
            	}
            }

            Column {
                spacing: 10 * main.vScaleFactor
                anchors {
                    left: parent.left
                    leftMargin: 0 * main.hScaleFactor
                    bottom: parent.bottom
                    bottomMargin: 0
                }
                visible: tvView.visible

//                Item {
//                    id: stopButton
//                    width: 53 * main.hScaleFactor
//                    height: 45 * main.vScaleFactor

//                    ScalableImage {
//                        anchors.fill: parent
//                        anchors.margins: 12 * main.scaleFactor
//                        fillMode: Image.PreserveAspectFit
//                        sourceName: "Stop_icon_WHITE";
//                    }

//                    MouseArea {
//                        anchors.fill: parent
//                        onClicked: {
//                            root.stop()
//                            root.action()
//                            buzzer.buttonPress()
//                        }

//                        Rectangle {
//                            anchors.fill: parent
//                            color: main.colorGenericStimulusArea
//                            opacity: 0.3
//                            visible: main.showStimulusArea
//                        }
//                    }
//                }

                Item {
                    id: sizeButton
                    width: 150 * main.hScaleFactor
                    height: 150 * main.vScaleFactor

                    ScalableImage {
                        width: 40 * main.hScaleFactor
                        height: 32 * main.vScaleFactor
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        sourceName: "UpperToolBar/expand_icon";
                    }

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            root.action()
                            fullscreenVideoFrame.showFullscreen(tv.player)
                            buzzer.buttonPress()
                        }

                        Rectangle {
                            anchors.fill: parent
                            color: main.colorGenericStimulusArea
                            opacity: 0.3
                            visible: main.showStimulusArea
                        }
                    }
                }
            }
        }
    }

    function channelSelected(id) {
        tv.on = true;
        print("TV channel selected id:" + id);

        if(id) {
            audioManager.tvRequest();
            root.channelID = id;
            tv.tuneChannelID(id);
        }
    }

    Component {
        id: menuButtonDelegate

        Item {
            id: menuButtonItem
            height: ListView.view.height * 0.2
            width: ListView.view.width

            property bool isCurrentItem: menuButtonItem.ListView.isCurrentItem
            property int _index: index

            function getChannelDesc()  {
                if (name || (number && name && number != name)) {
                    return name
                } else if (number) {
                    return number
                } else {
                    return ""
                }
            }

            Rectangle {
                color: menuButtonItem.isCurrentItem ? main.colorSelected : main.colorDialogBackground
                height: parent.height
                width: parent.width
                border.color: main.colorUIBorder
                border.width: main.widthBorder
                radius: main.defaultRadius
                anchors.horizontalCenter: parent.horizontalCenter
                
                Connections {
                    target: menuButtonMouseArea
                    ignoreUnknownSignals: true
                    onClicked: {
                        menuButtonItem.ListView.view.currentIndex = menuButtonItem._index;
                        root.action()
                    }
                }

                TextLabel {
                    anchors.fill: parent
                    anchors.margins: 12 * main.scaleFactor
                    horizontalAlignment: Text.AlignJustify
                    verticalAlignment: Text.AlignVCenter
                    text: getChannelDesc()
                    wrapMode: Text.WordWrap
                    font.pixelSize: 14 * main.vScaleFactor
                    clip: true
                }

                MouseArea {
                    id: menuButtonMouseArea
                    anchors.fill: parent
                    onClicked: {
                        channelSelected(id)
                        root.action()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }
    }

    ListModel {
        id: controlsList

        ListElement {
            name: "VolumeDown"
            label: QT_TR_NOOP("VOL")
            iconPath: "Media/Media_Volume_Minus"
        }
        ListElement {
            name: "VolumeUp"
            label: QT_TR_NOOP("VOL")
            iconPath: "Media/Media_Volume_Plus"
        }
        ListElement {
            name: "CloseCaption"
            label: ""
            iconPath: "Media/CC_icon"
        }
        ListElement {
            name: "VolumeMute"
            label: ""
            iconPath: "Media/Media_Mute_Up"
        }
    }

    ListModel {
        id: controlsListCab

        ListElement {
            name: "ChannelDown"
            label: qsTr("CHAN")
            iconPath: "Media/Media_Volume_Minus"
        }
        ListElement {
            name: "ChannelUp"
            label: qsTr("CHAN")
            iconPath: "Media/Media_Volume_Plus"
        }
        ListElement {
            name: "VolumeDown"
            label: qsTr("VOL")
            iconPath: "Media/Media_Volume_Minus"
        }
        ListElement {
            name: "VolumeUp"
            label: qsTr("VOL")
            iconPath: "Media/Media_Volume_Plus"
        }
        ListElement {
            name: "Keypad"
            label: ""
            iconPath: "Media/Remote_icon"
        }
        ListElement {
            name: "VolumeMute"
            label: ""
            iconPath: "Media/Media_Mute_Up"
        }
    }

    function controlClicked(name) {
        if(name === "VolumeDown") {
            volume_controls.volumeDown();
        } else if(name === "VolumeUp") {
            volume_controls.volumeUp();
        } else if(name === "CloseCaption") {
            tv.captionTrack = tv.numCaptionTracks > 0 ? (tv.captionTrack + 1) % tv.numCaptionTracks : 0
        } else if(name === "VolumeMute") {
            volume_controls.toggleMute();
        } else if(name === "ChannelDown") {
            if(root.tunedToCabChannel && options.cabEnabled)
                nucleus.cabChannelDown()
        } else if(name === "ChannelUp") {
            if(root.tunedToCabChannel && options.cabEnabled)
                nucleus.cabChannelUp()
        } else if(name === "Keypad") {
            if(options.cabEnabled) {
                /* Show keypad */
                channelKeypad.opacity = 1.0
            }
        }
    }

    Keypad{
        id:channelKeypad
        units: ""
        length: 3
        decimalPlaces: 1
        allow_close: false
        clear_after_enter: false
        show_entry: false
        icon: ""
        anchors.left: parent.left
        anchors.leftMargin: root.width * 0.20
        anchors.bottom: parent.bottom
        z: root.z + 1
        opacity: 0.0
        isColorInverted: true
        visible: (opacity > 0.0)
        labelForLeftArrow: "CC"

        onOpacityChanged: {
            if(opacity === 1.0) {
                timer.start()
            }
        }

        onKeyPress: {
            if(visible === false) {
                return;
            }

            timer.restart()

            switch(key) {
            case "0": nucleus.cabDigit(0); break;
            case "1": nucleus.cabDigit(1); break;
            case "2": nucleus.cabDigit(2); break;
            case "3": nucleus.cabDigit(3); break;
            case "4": nucleus.cabDigit(4); break;
            case "5": nucleus.cabDigit(5); break;
            case "6": nucleus.cabDigit(6); break;
            case "7": nucleus.cabDigit(7); break;
            case "8": nucleus.cabDigit(8); break;
            case "9": nucleus.cabDigit(9); break;
            case "<": nucleus.cabCloseCaption(); break;
            case ">":
                timer.stop()
                nucleus.cabEnter();
                channelKeypad.opacity = 0
                break;
            }
        }

        Timer {
            id: timer
            interval: 5000
            onTriggered: parent.opacity = 0
        }
    }
}
